//=============================================================================
// File: dynamic-clamp.c
// Func: calculate current values dynamically
// Copr: (C)2009 - 2010 by Frank Wuerriehausen, MPI bpc Goettingen
//	   http://
// Link:
// Load:
//=============================================================================


//-----------------------------------------------------------------------------
// includes

#include <stdio.h>
#include <avr32/io.h>
#include <string.h>
#include <ctype.h>
#include <math.h>
#include <time.h>

//#include <limits.h>

#include "compiler.h"
#include "board.h"
//#include "print_funcs.h"
#include "intc.h"
#include "spi_at32ap7000.h"
#include "pm_at32ap7000.h"
#include "gpio.h"
#include "usart.h"
#include "twi.h"
#include "smc_at32ap7000.h"
#include "sdramc_at32ap7000.h"
#include "tc.h"
#include "com.h"
#include "error.h"
#include "calc.h"
#include "utilities.h"
#include "wdt.h"
#include "at45dbx.h"
#include "global.h"
#include "cycle_counter.h"

#include "ctrl_access.h"
#include "fat.h"
#include "file.h"
#include "navigation.h"
#include "addrspace.h"

//#include "mmu.h"



extern void _firmwareloader(void);
extern void _initprog(void);
extern unsigned int	GetFlashID( void );
extern int	EraseSector( unsigned long addr );
extern int	FlashWrite( unsigned long addr, unsigned short value );
extern void	Test( unsigned long *addr, unsigned long, unsigned long, unsigned long );
extern void WriteFirmware( void );


extern char _start[];
extern char _etext[];
extern char _data_lma[];
extern char _data[];
extern char _end[];

//-----------------------------------------------------------------------------
// definitions


#define _MULTIPLIER			19				// 14 for 140 MHz, 15 for 150 MHz, 19 for 190 MHz

#define SPI_AD_BITS			16				// number of bits in each SPI package for AD and DA
#define SPI_AD_SPEED		18000000		// SPI speed in Hz for AD and DA

#define SPI_SF_BITS			8				// number of bits in each SPI package for serial flash
#define SPI_SF_SPEED		20000000		// SPI speed in Hz for serial flash

#define	SPI_NPCS0			0X0E
#define	SPI_NPCS1			0X0D
#define	SPI_NPCS2			0X0B

#define RTC_SPEED			50000			// TWI-Speed of the RTC
#define RTC_ADDRESS			0x68			// RTC's TWI address
#define RTC_ADDR_START		0				// Address start of the RTC
#define RTC_ADDR_LGT		1				// Address length of the RTC memory

#define TIMER_CHANNEL		0
#define TIMER_CLK_SRC		TC_CLOCK_SOURCE_TC2

#define	SUCCESS				0

#ifdef DBL
	#define DAC_SHIFT		0
	#define DAC_MASK		0x8FFF
	#define DAC_CHAN_SHIFT	15
//	#define RESOLUTION		4096.0
	#define VOLTAGE_SWING	2.048
//	#define VOLTAGE_SWING	8.192
//	#define MAX_VOLT		VOLTAGE_SWING / 2.0
//	#define MAX_VOLT		1.024
	#define MIN_INPUT		40			//   48 (-1V) - 8
	#define MAX_INPUT		4056		// 4048 (+1V) + 8
//	#define MIN_INPUT		48
//	#define MAX_INPUT		4048
#else
	#define DAC_SHIFT		2
	#define DAC_MASK		0x3FFC
//	#define RESOLUTION		4096.0
	#define VOLTAGE_SWING	2.0
//	#define MAX_VOLT		1.0
	#define MIN_INPUT		3
	#define MAX_INPUT		4092
#endif

#define RESOLUTION			4096.0
#define MAX_VOLT			VOLTAGE_SWING / 2.0
#define MAX_DAC_VALUE		0x0FFF

#define MIN_PERIOD			0.00002
#define MAX_PERIOD			1000
#define MIN_GAIN			0
#define MAX_GAIN			3
//#define MIN_SD				1e-300
#define MIN_SD				0
#define MAX_SD				1e+300
#define MIN_NSYN			1
#define MAX_NSYN			100

#define TRIGGER_DELAY		2				// trigger delay in us

#define CH0_SPI							(&AVR32_SPI0)


//-----------------------------------------------------------------------------
// prototypes

void Convert( void );
void MakeTimes( double t0, double dt0, long number, unsigned char method );
unsigned short ReadADC(  int nextChan );
int WriteDAC( unsigned short chan, unsigned short writeValue );
void GetDateString( time_t date, char *string );

//-----------------------------------------------------------------------------

//commands from serial line
enum
{
	HELP_CMD = 1,
	START_CMD,
	STOP_CMD,
	GAIN_CMD,
	READ_CMD,
	ECHO_CMD,
	GET_CMD,
	SET_CMD,
	INFO_CMD,
	BAUD_CMD,
	RESTART_CMD,
	DUMP_CMD,
	TEST_CMD,
	CLIP_CMD,
	GET_ERROR_CMD,
	VERSION_CMD,
	INIT_CMD,
	SETTRIGPOL_CMD,
	TIME_CMD,
	DATE_CMD,
	MAXTIME_CMD,
	INTERVAL_CMD,
	GMAX_CMD,
	TPEAK_CMD,
	EREV_CMD,
	GPEAK_CMD,
	TAUR_CMD,
	TAUD_CMD,
	ION_CMD,
	TP_CMD,
	GTP_CMD,
	DTTP_CMD,
	ETP_CMD,
	T0TP_CMD,
	GABA_ALPHA_CMD,
	GABA_2EXP_CMD,
	DYNAMIC_CMD,
	NSITE_CMD,
    PROB_CMD,
	SIGMA_CMD,
	TAUREC_CMD,
	TAUF_CMD,
	RANDS_CMD,
	GUNIT_CMD,
	GDYNAMIC_CMD,
	GSD_CMD,
	MAKELT_CMD,
	AMPA_CMD,
	NSYN_CMD,
	SETTINGS_CMD,
	FILE_CMD,
	OPEN_CMD,

	CD_CMD,
	MOUNT_CMD,
	CP_CMD,
	LS_CMD,
	RM_CMD,
	DF_CMD,
	MKDIR_CMD,
	TOUCH_CMD,
	APPEND_CMD,
	CAT_CMD,
	NB_DRIVE_CMD,
	SET_ID_CMD,
	GOTO_ID_CMD,
	MV_CMD,
	FORMAT_CMD,
	FORMAT32_CMD,
	FAT_CMD,
	FIRMWARE_CMD,
	WRITE_FLASH_CMD,
	LED_CMD,
	CHECK_FW_CMD,
	OFFSET_CMD,
	DIAGNOSTIC_CMD,
	MAX_CMD,
};


//time calculation methodes
enum
{
	T_MANUAL = 1,
	T_REGULAR,
	T_POISSON
};


//-----------------------------------------------------------------------------
// structures

typedef struct
{
	unsigned long		trigger;		// number of trigger
	unsigned long		time;			// time of occurence
} clipStruct;


struct dcSettingsRecord
{
	unsigned long		ident;
	unsigned long		serno;
	int					version;
	int					gain;
	int					ionChannel;

	double				interval;		// 20us, 50KHz
	double				timeEnd;		// sec
	double				tPeak;			// sec
	double				eRev;

	double				gMax;			// Siemens;

	double				pr;				// release probability
	unsigned long		numRS;			// number of release site
	double				sigma;			// standard deviation of PSR
	double				taurec;			// seconds;
    double				tauf;
	double				gUnit;

	double				sd;
	unsigned long		nSyn;			// the number of synapse

	double				gPeak;
	double				tauR;
	double				tauD;
	double				gTP;
	double				eTP;
	unsigned long		dtTP;
	unsigned long		t0TP;

	double				t0;
	double				dt0;
	unsigned long		numT0;

	unsigned char		tMethode;
	unsigned char		space1;
	unsigned char		space2;
	unsigned char		space3;

	long				date;
	short				offset[2];
//	long				space4;

	double				space5[15];
};
typedef	struct dcSettingsRecord dcSettingsRecord;
typedef	struct dcSettingsRecord *dcSettingsRecordPtr;
typedef	struct dcSettingsRecord *dcSettings;




//-----------------------------------------------------------------------------
// constant strings

const char		*commands[] =
	{
		"help", 			// HELP_CMD
		"start",			// START_CMD
		"stop",				// STOP_CMD
		"gain",				// GAIN_CMD
		"read",				// READ_CMD
		"echo",				// ECHO_CMD
		"get",				// GET_CMD
		"set",				// SET_CMD
		"info",				// INFO_CMD
		"baud",				// BAUD_CMD
		"restart",			// RESTART_CMD
		"dump",				// DUMP_CMD
		"test",				// TEST_CMD
		"clip",				// CLIP_CMD
		"geterror",			// GET_ERROR_CMD
		"version",			// VERSION_CMD
		"init",				// INIT_CMD
		"trigpol",			// SETTRIGPOL_CMD
		"time",				// TIME_CMD
		"date",				// DATE_CMD
		"maxtime",			// MAXTIME_CMD
		"interval",			// INTERVAL_CMD
		"gmax",				// GMAX_CMD
		"tpeak",			// TPEAK_CMD
		"erev",				// EREV_CMD
		"gpeak",			// GPEAK_CMD
		"taur",				// TAUR_CMD
		"taud",				// TAUD_CMD
		"ion",				// ION_CMD
		"testpulse",		// TP_CMD
		"gtp",				// GTP_CMD
		"dttp",				// DTTP_CMD
		"etp",				// ETP_CMD
		"t0tp",				// T0TP_CMD
		"gaba_alpha",		// GABA_ALPHA_CMD
		"gaba_2exp",		// GABA_2EXP_CMD
		"dynamic",			// DYNAMIC_CMD
		"nsite",			// NSITE_CMD
		"prob",				// PROB_CMD
		"sigma",			// SIGMA_CMD
		"taurec",			// TAUREC_CMD
		"tauf",             // TAUF_CMD
		"rands",			// RANDS_CMD
		"gunit",			// GUNIT_CMD
		"gdynamic",			// GDYNAMIC_CMD
		"gsd",				// GSD_CMD
		"makelt",			// MAKELT_CMD
		"ampa",				// AMPA_CMD
		"nsyn",				// NSYN_CMD
		"settings",			// SETTINGS_CMD
		"file",				// FILE_CMD
		"open",				// OPEN_CMD

		"cd",				// CD_CMD
		"mount",			// MOUNT_CMD
		"cp",				// CP_CMD
		"ls",				// LS_CMD
		"rm",				// RM_CMD
		"df",				// DF_CMD
		"mkdir",			// MKDIR_CMD
		"touch",			// TOUCH_CMD
		"append",			// APPEND_CMD
		"cat",				// CAT_CMD
		"disk",				// NB_DRIVE_CMD
		"mark",				// SET_ID_CMD
		"goto",				// GOTO_ID_CMD
		"mv",				// MV_CMD
		"format",			// FORMAT_CMD
		"format32",			// FORMAT32_CMD
		"fat",				// FAT_CMD
		"firmware",			// FIRMWARE_CMD
		"flash",			// WRITE_FLASH_CMD
		"led",				// LED_CMD
		"checkfw",			// CHECK_FW_CMD
		"offset",			// OFFSET_CMD
		"diagnostic",		// DIAGNOSTIC_CMD
		NULL
	};



const char		*gainStrings[] =
	{
		"0.1 pA/mV",
		"1 pA/mV",
		"10 pA/mV",
		"100 pA/mV"
	};

const char		*ionChannels[] = 	{ "testpulse", "gaba_alpha", "gaba_2exp", "ampa", "dynamic", NULL };
const char		*timeMethods[] = 	{ "undefined", "manual", "regular", "poisson", NULL };
const char		*switches[] = 		{ "off", "on", NULL };


//-----------------------------------------------------------------------------
// global variables

volatile avr32_spi_t	*spi = &AVR32_SPI0;
volatile unsigned long	g1msCounter = 1UL, g100msCounter = 1UL;
volatile unsigned long	gTimeCnt = 0UL;
unsigned long			gVal1ms;
unsigned long			gSpiADSpeed = SPI_AD_SPEED, gSpiSFSpeed = SPI_SF_SPEED;
//long					gUsecTime;
//char					gMessage[100];
char					gMFlag = 0, gStartFlag = 0, gStopFlag = 0, gTriggerFlag = 0;
char					gClip = 0, gRunFlag = 0, gPauseFlag = 0, gFirstFlag = 1, gMltFlag = 1;
char					gEcho = 1, firmwareFlag = 0, gDiagnosticFlag = 1;
//char					gIntCnt = 0, gTriggerPolarity = 0;
volatile char			gHeartbeat = 0;
long					gTimerValue, gTimerValue1;
unsigned long			gClipCounter = 0UL, gLastClipTime = 0UL, gTriggerCounter = 0UL;
//long					gRands;						// random number seed

long					gDate = 0;
volatile unsigned long	gMilliSeconds = 0UL;

dcSettingsRecord		gSettings;

unsigned long			gNumValues, gFirstValue, gNumTimeSteps, gFirstFreeRam;
//unsigned long			gNumT0 = 0L;

static char				gDataFile[MAX_FILE_PATH_LENGTH + 1];


//! flag for first ls : mount if set
static Bool				first_ls;
//! string length
//static U8				i_str = 0;
//! buffer for command line
static char				str_buff[MAX_FILE_PATH_LENGTH + 1];
static char				par_str1[MAX_FILE_PATH_LENGTH + 1];

volatile clipStruct		*clipTimePtr = NULL;
double					*gT0 = NULL;


//-----------------------------------------------------------------------------
// functions

/*  the TIMER interrupt handler.
 *
 *  \note The `__attribute__((__interrupt__))'
 *  attribute is used to manage the `rete' instruction.
 */
__attribute__((__interrupt__))
static void TimerIntHandler(void)
{
	volatile avr32_pio_t			*pioa	= &AVR32_PIOA;
	volatile avr32_tc_t				*tc		= &AVR32_TC0;
//	int								dummy;

#ifdef DEBUG
	pioa->sodr	=	(1 << AVR32_PIO_P23);
/*
*/
	if (pioa->pdsr & (1 << AVR32_PIO_P22))
	{
		pioa->codr	=	(1 << AVR32_PIO_P22);	// set output port to low
	}
	else
	{
		pioa->sodr	=	(1 << AVR32_PIO_P22);	// set output port to high
	}
#endif

	if (gTriggerFlag)
	{
		tc_write_rc( tc, TIMER_CHANNEL, gTimerValue );		// set the correct value back to the timer
		gTriggerFlag = 0;
	}

	tc_read_sr( tc, TIMER_CHANNEL );						//
	gTimeCnt++;

	if (gStopFlag) gStartFlag = 0;

	if (gMFlag && gStartFlag)
	{
		Convert();
	}


	if (gStartFlag && gRunFlag == 0 && (gTimeCnt >= gNumTimeSteps))
	{
		gStartFlag = 0;
		gPauseFlag = 1;
//		dummy = pioa->isr;
//		pioa->ier	=	  (1 << AVR32_PIO_P21);		// enable interupt
//		testFlag = 0;
	}

	g1msCounter--;
	if (g1msCounter == 0 || g1msCounter > gVal1ms)
	{
		g1msCounter = gVal1ms;
		g100msCounter--;
		// update the counters for milliseconds and seconds
		if (++gMilliSeconds == 1000)
		{
			gMilliSeconds = 0;
			gDate++;
		}
	}

	if (g100msCounter == 0)
	{
		g100msCounter = 100;

		// make a heartbeat on the sys-LED
		switch (gHeartbeat++)
		{
		case 0:
			pioa->codr	=	(1 << AVR32_PIO_P16);	// set output to low (LED on)
			break;

		case 1:
			pioa->sodr	=	(1 << AVR32_PIO_P16);	// set output to high (LED off)
			break;

		case 3:
			pioa->codr	=	(1 << AVR32_PIO_P16);	// set output to low (LED on)
			break;

		case 4:
			pioa->sodr	=	(1 << AVR32_PIO_P16);	// set output to high (LED off)
			break;

		case 9:
			gHeartbeat	 = 0;
			break;
		}
	}

#ifdef DEBUG
	pioa->codr	=	(1 << AVR32_PIO_P23);
#endif
}

/*  the PIOA interrupt handler (trigger).
 *
 *  \note The `__attribute__((__interrupt__))'
 *  attribute is used to manage the `rete' instruction.
 */

__attribute__((__interrupt__))
static void PIOAIntHandler(void)
{
	volatile avr32_tc_t				*tc			= &AVR32_TC0;
	volatile avr32_pio_t			*pioa		= &AVR32_PIOA;
//	volatile avr32_pio_t			*pioe		= &AVR32_PIOE;
	int								pioaIsr;

	pioaIsr = pioa->isr;

	if	(	gMFlag
		&& 	!gStartFlag
		&&	(pioaIsr & (1 << AVR32_PIO_P21))
//		&&	!((pioa->pdsr & (1 << AVR32_PIO_P21)) ^ (gTriggerPolarity << AVR32_PIO_P21))
		)
	{
		pioa->idr	=	  (1 << AVR32_PIO_P21);		// disable interupt on trigger line
		tc_software_trigger( tc, TIMER_CHANNEL );			// trigger the timer and set it to 0
		tc_read_sr( tc, TIMER_CHANNEL );						//
		tc_write_rc( tc, TIMER_CHANNEL, gTimerValue1 );		// the first timer value is corrected by the trigger delay
		gStartFlag = 1;
		gTriggerFlag = 1;
		gTimeCnt = 0UL;
		gTriggerCounter++;
		gSettings.date = gDate;
	}
	else if (pioaIsr & (1 << AVR32_PIO_P31))		// for loading the cache only
	{
		pioa->idr	=	  (1 << AVR32_PIO_P31);		// disable interupt
		if (gFirstFlag)
		{
			gFirstFlag = 0;
			tc_software_trigger( tc, TIMER_CHANNEL );
//			gStartFlag = 1;
			gTimeCnt = 0UL;
			Convert();
			gFirstFlag = 0;
		}
	}

	pioaIsr = pioa->isr;
}

//-----------------------------------------------------------------------------
void DummyInt( void )
{
	volatile avr32_pio_t			*pioa		= &AVR32_PIOA;

	// generate an interupt to fill the cache with the right program parts
	// otherwise the first cycle will take too long
	// port PA30 and PA31 are connected
	pioa->ier	=	  (1 << AVR32_PIO_P31);		// enable interupt

	// change the level on port 30 to generate an interupt on port 31
	if (pioa->pdsr & (1 << AVR32_PIO_P30))
	{
		pioa->codr	=	(1 << AVR32_PIO_P30);	// set output port to low
	}
	else
	{
		pioa->sodr	=	(1 << AVR32_PIO_P30);	// set output port to high
	}
}


/*  Read the DAC, calculate the current, and write to the ADC
 *
 */
void Convert( void )
{
//	int						error = 0;
	unsigned short			readValue, writeValue = 0;
	double					current = 0.0, voltage/*, time, timet*/;

	readValue = ReadADC( 0 );

/*
	if (readValue < MIN_INPUT || readValue > MAX_INPUT)
	{
//		gClip = 1;
		if ((gClipCounter == 0UL) || ((gTimeCnt - 1UL) > gLastClipTime))
		{
			clipTimePtr[gClipCounter].trigger = gTriggerCounter;
			clipTimePtr[gClipCounter].time = gTimeCnt;
			gClipCounter++;
		}
		gLastClipTime = gTimeCnt;
	}
*/

	voltage = ((double)readValue / RESOLUTION * VOLTAGE_SWING - MAX_VOLT) / 10.0;

	if (gSettings.ionChannel == kTp)
	{
		if (gTimeCnt >= gFirstValue)
		{
			if (gTimeCnt < gNumValues)
			{
				current = gSettings.gTP * (voltage - gSettings.eTP);
				switch (gSettings.gain)
				{
					case 0:
						current /= 1.0e-10;
						break;
					case 1:
						current /= 1.0e-9;
						break;
					case 2:
						current /= 1.0e-8;
						break;
					case 3:
						current /= 1.0e-7;
						break;
				}
			}
		}
	}
	else
	{
		current = CalcI( gTimeCnt, voltage, gSettings.eRev, gFirstValue, gNumValues );
	}
	if (current < -1.0 || current > 1.0) gClip = 1;
	writeValue = ((unsigned short)((current + MAX_VOLT) * (RESOLUTION / VOLTAGE_SWING)));
	WriteDAC( 0, writeValue );
}

void InitVariables( void )
{
	gDate = 0;
	g1msCounter = 1UL;
	g100msCounter = 1UL;
	gMilliSeconds = 0UL;
	gHeartbeat = 0;
	gTimeCnt = 0UL;
	gMFlag = 0;
	gStartFlag = 0;
	gStopFlag = 0;
	gTriggerFlag = 0;
	gClip = 0;
	gRunFlag = 0;
	gPauseFlag = 0;
	gFirstFlag = 1;
	gMltFlag = 1;
	gEcho = 1;
	firmwareFlag = 0;
	gClipCounter = 0UL;
	gLastClipTime = 0UL;
	gTriggerCounter = 0UL;
	gSettings.interval = 0.00002;
//	clipTimePtr = NULL;
//	gT0 = NULL;
}
//-----------------------------------------------------------------------------
void SaveTimes( void )
{
	unsigned char	*tmpPtr = (void*)gT0;
	int				i;

	// Try to to set navigator on times file.
	if (!nav_setcwd((FS_STRING)TIMES_FILE, TRUE, TRUE))
	{
		// Display error message.
		PrintError( "", ERR_UNKNOWN_FILE );
		return;
	}

	// File exists, open it in write mode
	file_open( FOPEN_MODE_W );
	// Append from USART
	for (i = gSettings.numT0 * sizeof( double ); i; i--)
	{
		// Add it to the file.
		file_putc( *tmpPtr++ );
	}
	// Close the file
	file_close();
}

/*
//-----------------------------------------------------------------------------
int LoadTimes( void )
{
	unsigned char	*tmpPtr;
	int				i;

	if (gT0 == NULL)
	{
		if ( ( gT0 = malloc( sizeof( double ) * gSettings.numT0 ) ) == NULL )
		{
			gSettings.numT0 = 0;
			return NO_MEMORY;
		}
	}
	else
	{
		if ( ( gT0 = realloc( gT0, sizeof( double ) * gSettings.numT0 ) ) == NULL )
		{
			gSettings.numT0 = 0;
			return NO_MEMORY;
		}
	}

	tmpPtr = (void*)gT0;
	// Try to to set navigator on settings file.
	if (!nav_setcwd((FS_STRING)TIMES_FILE, TRUE, FALSE))
	{
		return ERR_UNKNOWN_FILE;
	}
	else
	{
		// Open the file.
		file_open(FOPEN_MODE_R);
		// read the settings
		for (i = gSettings.numT0 * sizeof( double ); i; i--)
		{
			if (file_eof()) break;
			// get next char from file.
			*tmpPtr++ = file_getc();
		}
		// Close the file
		file_close();
	}

	return SUCCESS;
}
*/

//-----------------------------------------------------------------------------
int LoadTimes( void )
{
	unsigned char	*tmpPtr;
	int				i;
	unsigned long	len;
//	char			string1[30];

	gSettings.numT0 = 0;

	// Try to to set navigator on settings file.
	if (!nav_setcwd((FS_STRING)TIMES_FILE, TRUE, FALSE))
	{
		return ERR_UNKNOWN_FILE;
	}
	else
	{
		len = nav_file_lgt();

		if (gT0 == NULL)
		{
			if ((gT0 = malloc( len )) == NULL) return NO_MEMORY;
		}
		else
		{
			if ((gT0 = realloc( gT0, len )) == NULL) return NO_MEMORY;
		}

		tmpPtr = (void*)gT0;

		// Open the file.
		file_open( FOPEN_MODE_R );
		// read the settings
		for (i = len; i; i--)
		{
			if (file_eof()) break;
			// get next char from file.
			*tmpPtr++ = file_getc();
		}
		// Close the file
		file_close();
		gSettings.numT0 = (len - i) / sizeof( double );

/*
		Printf( "numT0 = %ul\n", gSettings.numT0 );
		for (i = 0; i < gSettings.numT0; i++)
		{
			Dbl2Str( string1, gT0[i] );
			Printf( "t0[%d] = %ss\n", i, string1 );
//			Printf( "gT0[%d] = %g\n", i, gT0[i] );
		}
*/
	}

	return SUCCESS;
}


//-----------------------------------------------------------------------------
int SaveSettings( void )
{
	unsigned char	*tmpPtr = (void*)&gSettings;
	int				i;

	// Try to to set navigator on settings file.
	if (!nav_setcwd((FS_STRING)SETTINGS_FILE, TRUE, TRUE))
	{
		return ERR_UNKNOWN_FILE;
	}

	// File exists, open it in write mode
	file_open( FOPEN_MODE_W );
	// Append from USART
	for (i = 0; i < sizeof( dcSettingsRecord ); i++)
	{
		// Add it to the file.
		file_putc( *tmpPtr++ );
	}
	// Close the file
	file_close();

	if (gSettings.tMethode == T_MANUAL )
	{
		SaveTimes();
	}

	return SUCCESS;
}

//-----------------------------------------------------------------------------
int LoadSettings( void )
{
	unsigned char	*tmpPtr = (void*)&gSettings;
	int				i, error = 0;

	// Try to to set navigator on settings file.
	if (!nav_setcwd((FS_STRING)SETTINGS_FILE, TRUE, FALSE))
	{
		// Display error message.
//		PrintError( SETTINGS_FILE, ERR_UNKNOWN_FILE );
		error = ERR_UNKNOWN_FILE;
	}
	else
	{
		// Open the file.
		file_open(FOPEN_MODE_R);
		// read the settings
		for (i = 0; i < sizeof( dcSettingsRecord ); i++)
		{
			if (file_eof()) break;
			// get next char from file.
			*tmpPtr++ = file_getc();
		}
		// Close the file
		file_close();
	}

	return error;
}

//-----------------------------------------------------------------------------
void InitSettings( int init )
{
//	unsigned char	*tmpPtr = (void*)&gSettings;
	int				error = 0;

	gSettings.ident = 0;

	if (!init)
	{
		error = LoadSettings();
		if (!error)
		{
			Prints( " settings loaded\n" );
		}
	}

	if (error || gSettings.ident != IDENT)
	{
		gSettings.ident = IDENT;
		gSettings.version = VERSION;
		gSettings.gain = 1;
		gSettings.ionChannel = 1;

		gSettings.interval = 0.00002;		// 20us, 50KHz
		gSettings.timeEnd = 0.25;			// sec
		gSettings.tPeak = 0.001;			// sec
		gSettings.eRev = -0.07;

		gSettings.gMax = 0.5e-9;			// Siemens;

		gSettings.pr = 0.05;				// release probability
		gSettings.numRS = 100;				// number of release site
		gSettings.sigma = 0.4;				// standard deviation of PSR
		gSettings.taurec = 4.7;				// seconds;
        gSettings.tauf = 0.01;               // seconds
		gSettings.gUnit = 1.0e-9;

		gSettings.sd = 1.0e-9;
		gSettings.nSyn = 1;					// the number of synapse

		gSettings.gPeak = 0.05e-9;
		gSettings.tauR = 0.001;
		gSettings.tauD = 0.01;
		gSettings.gTP = 0.0;
		gSettings.eTP = 0.0;
		gSettings.dtTP = 0UL;
		gSettings.t0TP = 0UL;

		gSettings.t0 = 0.0;
		gSettings.dt0 = 0.0;
		gSettings.numT0 = 0;

		gSettings.tMethode = 0;

		gSettings.offset[0] = 0;
		gSettings.offset[1] = 0;

		Prints( " settings initialized\n" );
	}

	if (gSettings.tMethode == T_MANUAL)
	{
		error = LoadTimes();
		if (!error)
		{
			Prints( " time data loaded\n" );
		}
	}
}


//-----------------------------------------------------------------------------
void CalcTimerValues( void )
{
	gTimerValue = (pm_read_module_freq_hz( PM_PBB_TIMER0 ) / 1000000		// value for period
			* (long)(gSettings.interval * 1000000.0)) / (1 << (TIMER_CLK_SRC + 1));
	gTimerValue1 = (pm_read_module_freq_hz( PM_PBB_TIMER0 ) / 1000000		// value for period - trigger delay
			* ((long)(gSettings.interval * 1000000.0) - TRIGGER_DELAY)) / (1 << (TIMER_CLK_SRC + 1));

//	gVal100ms = (unsigned long)(1.0 / gSettings.interval) / 10;
	gVal1ms = (unsigned long)(1.0 / gSettings.interval + 0.5) / 1000;
//	Printf( " gVal1ms = %ld\n", gVal1ms );
}


//-----------------------------------------------------------------------------
void InitTimer( void )
{
	volatile avr32_tc_t				*tc		= &AVR32_TC0;

	// Options for waveform genration.
	tc_waveform_opt_t				waveform_opt =
	{
		.channel  = TIMER_CHANNEL,							// Channel selection.

		.bswtrg		= TC_EVT_EFFECT_NOOP,					// Software trigger effect on TIOB.
		.beevt		= TC_EVT_EFFECT_NOOP,					// External event effect on TIOB.
		.bcpc		= TC_EVT_EFFECT_NOOP,					// RC compare effect on TIOB.
		.bcpb		= TC_EVT_EFFECT_NOOP,					// RB compare effect on TIOB.

		.aswtrg		= TC_EVT_EFFECT_NOOP,					// Software trigger effect on TIOA.
		.aeevt		= TC_EVT_EFFECT_NOOP,					// External event effect on TIOA.
		.acpc		= TC_EVT_EFFECT_NOOP,					// RC compare effect on TIOA.
		.acpa		= TC_EVT_EFFECT_NOOP,					// RA compare effect on TIOA.

		.wavsel		= TC_WAVEFORM_SEL_UP_MODE_RC_TRIGGER,			// Waveform selection: Up mode without automatic trigger on RC compare.
		.enetrg		= FALSE,								// External event trigger enable.
		.eevt		= TC_EXT_EVENT_SEL_TIOB_INPUT,			// External event selection.
		.eevtedg	= TC_SEL_NO_EDGE,						// External event edge selection.
		.cpcdis		= FALSE,								// Counter disable when RC compare.
		.cpcstop	= FALSE,								// Counter clock stopped with RC compare.

		.burst		= TC_BURST_NOT_GATED,					// Burst signal selection.
		.clki		= TC_CLOCK_RISING_EDGE,					// Clock inversion.
		.tcclks		= TIMER_CLK_SRC							// Internal source clock 2, connected to fPBB / 4.
	};

	tc_interrupt_t					timerInt =
	{
		.etrgs	= 0,			// External trigger interrupt
		.ldrbs	= 0,			// RB load interrupt
		.ldras	= 0,			// RA load interrupt
		.cpcs	= 1,			// RC compare interrupt
		.cpbs	= 0,			// RB compare interrupt
		.cpas	= 0,			// RA compare interrupt
		.lovrs	= 0,			// Load overrun interrupt
		.covfs	= 0				// Timer counter overflow interrupt
	};



	CalcTimerValues();

	tc_init_waveform( tc, &waveform_opt );
	tc_write_rc( tc, TIMER_CHANNEL, gTimerValue );
	tc_configure_interrupts( tc, TIMER_CHANNEL, &timerInt );
	tc_start( tc, TIMER_CHANNEL );
}

//-----------------------------------------------------------------------------
int InitTimeValues( void )
{
	int		error = 0;

	clipTimePtr	= 	(void *) (LOOKUP_TABLE);

	if (gSettings.numT0 == 0)
	{
		gSettings.numT0 = 2;

		if (gT0 == NULL)
		{
			if ( ( gT0 = malloc( sizeof( double ) * gSettings.numT0 ) ) == NULL )
			{
				gSettings.numT0 = 0;
				return NO_MEMORY;
			}
		}
		else
		{
			if ( ( gT0 = realloc( gT0, sizeof( double ) * gSettings.numT0 ) ) == NULL )
			{
				gSettings.numT0 = 0;
				return NO_MEMORY;
			}
		}
		gT0[0] = 0.02;
		gT0[1] = 0.04;


		gNumValues = (long)((gSettings.timeEnd - gT0[0]) / gSettings.interval + 0.5);
		gFirstValue = (long)(gT0[0] / gSettings.interval + 0.5);
		gFirstFreeRam = (gNumValues + 1UL) * sizeof( double );
		clipTimePtr	= 	(void *) (LOOKUP_TABLE + gFirstFreeRam);
		gClipCounter = 0UL;
		gTriggerCounter = 0UL;

		gSettings.tMethode = T_MANUAL;

		if( ( gDGMax = malloc( sizeof( double ) * gSettings.numT0 ) ) == NULL )
		{
			return NO_MEMORY;
		}
		gDGMax[0] = 1;
		gDGMax[1] = 2;

		error = SaveSettings();
	}
	else
	{
/*
		if (gT0 == NULL)
		{
			if ((gT0 = malloc( sizeof( double ))) == NULL)
			{
				gSettings.numT0 = 0;
				return NO_MEMORY;
			}
		}
*/
		MakeTimes( gSettings.t0, gSettings.dt0, gSettings.numT0, gSettings.tMethode );
	}

	gNumTimeSteps = (long)(gSettings.timeEnd / gSettings.interval + 0.5);

	return error;
}

//-----------------------------------------------------------------------------
void InitPIO( void )
{
	volatile avr32_pio_t			*pioa		= &AVR32_PIOA;
	volatile avr32_pio_t			*pioe		= &AVR32_PIOE;


	pioa->per	=	  (1 << AVR32_PIO_P16)		// enable pio
					| (1 << AVR32_PIO_P19)
					| (1 << AVR32_PIO_P21)
					| (1 << AVR32_PIO_P22)
					| (1 << AVR32_PIO_P23)
					| (1 << AVR32_PIO_P30)
					| (1 << AVR32_PIO_P31);

	pioa->oer	=	  (1 << AVR32_PIO_P16)		// set port as output
					| (1 << AVR32_PIO_P19)
					| (1 << AVR32_PIO_P22)
					| (1 << AVR32_PIO_P23)
					| (1 << AVR32_PIO_P30);

	pioa->pudr	=	  (1 << AVR32_PIO_P16)		// disable pull ups
					| (1 << AVR32_PIO_P19)
					| (1 << AVR32_PIO_P22)
					| (1 << AVR32_PIO_P23)
					| (1 << AVR32_PIO_P30);

	pioa->sodr	=	  (1 << AVR32_PIO_P16)		// set output to high (LED off)
					| (1 << AVR32_PIO_P19);

	pioa->codr	=	  (1 << AVR32_PIO_P22)		// set output to low
					| (1 << AVR32_PIO_P23)
					| (1 << AVR32_PIO_P30);

	pioa->ifer	=	  (1 << AVR32_PIO_P21)		// enable glitch input filter
					| (1 << AVR32_PIO_P31);
	pioa->puer	=	  (1 << AVR32_PIO_P21)		// enable pull up
					| (1 << AVR32_PIO_P31);
//	pioa->ier	=	  (1 << AVR32_PIO_P21);		// enable interupt


	pioe->per	=	  (1 << AVR32_PIO_P19);		// enable pio
	pioe->oer	=	  (1 << AVR32_PIO_P19);		// set port as output
	pioe->pudr	=	  (1 << AVR32_PIO_P19);		// disable pull ups
	pioe->sodr	=	  (1 << AVR32_PIO_P19);		// set output to high (LED off)
}



/*  initialise SPI in master mode
 *
 *  \param spi Pointer to the correct avr32_spi_t struct
 *  \param cpuHz the clock frequency in Hz of the SPI module
 */
unsigned char GetBaudRate( unsigned long requestedBaud, long cpuHz, unsigned long *usedBaud, char *string )
{
	unsigned long	baud, result;
	int				scale;
	unsigned char	div = 0;
	char			fUnit[4];

	baud = cpuHz / requestedBaud + 1;
	result = cpuHz / baud;

	if (baud > 255)
	{
		baud /= 32;
		div = 1;
		result = cpuHz / baud / 32;
	}

	if (result > 1000000)
	{
		scale = 6;
		strcpy( fUnit, "MHz" );
	}
	else if (result > 1000)
	{
		scale = 3;
		strcpy( fUnit, "kHz" );
	}
	else
	{
		scale = 0;
		strcpy( fUnit, "Hz" );
	}

	sprintf( string, "%ld %s", result, fUnit );
	DivString( scale, 3, string );

	*usedBaud = result;
	return div;
}



/*  initialise SPI in master mode
 *
 *  \param spi Pointer to the correct avr32_spi_t struct
 *  \param cpuHz the clock frequency in Hz of the SPI module
 */
int InitSpi( volatile avr32_spi_t *spi, long cpuHz, unsigned long baudrate )
{
	spi_options_t	spiOptions;
	unsigned long	result;
	int				error = 0;
	char			str1[20];

	spiOptions.fdiv = GetBaudRate( baudrate, cpuHz, &result, str1 );
//	Printf( " SPI AD/DA baudrate: %s\n", str1 );

	spiOptions.modfdis = 1;

	// Initialize SPI as master
	if ((error = spi_initMaster( spi, &spiOptions ))) return error;

	// Set master mode; variable_ps, pcs_decode, delay
	error = spi_selectionMode( spi, 1, 0, 0 );

	return error;
}



/*  initialise SPI in master mode
 *
 *  \param spi Pointer to the correct avr32_spi_t struct
 *  \param cpuHz the clock frequency in Hz of the SPI module
 */
int InitSpiMaster( volatile avr32_spi_t *spi, long cpuHz )
{
	spi_options_t	spiOptions;
	unsigned long	error = 0, result;
	char			str1[20];

	if ((error = InitSpi( spi, cpuHz, gSpiADSpeed ))) return error;
//	error = InitSpi( spi, cpuHz, gSpiADSpeed );
//	Printf( "error1 = %lu\n", error );
//	if (error) return error;


	GetBaudRate( gSpiADSpeed, cpuHz, &result, str1 );

	spiOptions.baudrate = result;
	spiOptions.bits = SPI_AD_BITS;
	spiOptions.spck_delay = 8;
	spiOptions.trans_delay = 0;
	spiOptions.stay_act = 0;

	// setup CS1 (ADC) in mode 0
	spiOptions.reg = 1;
#ifdef DBL
	spiOptions.spi_mode = 0;
#else
	spiOptions.spi_mode = 0;
#endif
	if ((error = spi_setupChipReg( spi, &spiOptions, cpuHz ))) return error;
//	error = spi_setupChipReg( spi, &spiOptions, cpuHz );
//	Printf( "error2 = %lu, baudrate = %lu, reg = %u, bits = %u, stay_act = %u, mode = %u, trans_delay = %u\n",
//				error, result, spiOptions.reg, spiOptions.bits, spiOptions.stay_act, spiOptions.spi_mode, spiOptions.trans_delay );
//	if (error) return error;

	// setup CS2 (DAC) in mode 3
	spiOptions.reg = 2;
#ifdef DBL
	spiOptions.spi_mode = 0;
#else
	spiOptions.spi_mode = 3;
#endif
	if ((error = spi_setupChipReg( spi, &spiOptions, cpuHz ))) return error;
//	error = spi_setupChipReg( spi, &spiOptions, cpuHz );
//	Printf( "error3 = %lu\n", error );
//	if (error) return error;


	GetBaudRate( gSpiSFSpeed, cpuHz, &result, str1 );
//	Printf( " SPI SF baudrate: %s\n", str1 );

	spiOptions.baudrate = result;
	spiOptions.bits = SPI_SF_BITS;
	spiOptions.spck_delay = 0;
	spiOptions.trans_delay = 0;
	spiOptions.stay_act = 1;
	spiOptions.spi_mode = 0;
	spiOptions.reg = 0;

	// setup CS0 (serial flash) in mode 3
	if ((error = spi_setupChipReg( spi, &spiOptions, cpuHz ))) return error;


	spi_enable( spi );

	return error;
}


/*****************************************************************
* Function:	 ConvertToLower()
* Arguments:	char *buf a char pointer to a buffer
* Returns:	  none
******************************************************************/

void ConvertToLower( char *buf )
{
	while (*buf)
	{
		if ( isupper( *buf ) )
		{
			*buf = tolower( *buf );
		}
		buf++;
	}
}


//-----------------------------------------------------------------------------
unsigned short ReadADC(  int nextChan )
{
//	int				error;
	unsigned short	readValue, chan = 0;

	if (nextChan == 1) chan = 0xFFFF;
//	spi_empty_rd( spi );
#ifdef DBL
//	error = spi_variableSlaveWrite( spi, chan, SPI_NPCS2, 1 );
	spi_variableSlaveWrite( spi, chan, SPI_NPCS1, 1 );
#else
	spi_variableSlaveWrite( spi, chan, SPI_NPCS1, 1 );
#endif
//	if (error) PrintError( "ERROR while starting the ADC", error );

	spi_read( spi, &readValue );
//	error =  spi_read( spi, &readValue );
//	if (error) PrintError( "ERROR while reading from the ADC", error );

	return readValue & 0xFFF;
}

//-----------------------------------------------------------------------------
int WriteDAC( unsigned short chan, unsigned short writeValue )
{
	int				error;

//	if (writeValue > MAX_DAC_VALUE) writeValue = MAX_DAC_VALUE;
	writeValue -= gSettings.offset[chan];
	writeValue &= MAX_DAC_VALUE;

#ifdef DBL
//	writeValue = (writeValue | (chan << DAC_CHAN_SHIFT)) & DAC_MASK;
	writeValue = (writeValue | (chan << DAC_CHAN_SHIFT));
#else
//	writeValue = (writeValue << DAC_SHIFT) & DAC_MASK;
	writeValue = (writeValue << DAC_SHIFT);
#endif

//	Printf( " DACvalue = %04X\n", writeValue );

//#ifdef DBL
	error = spi_variableSlaveWrite( spi, writeValue, SPI_NPCS2, 1 );
//#else
//	error = spi_variableSlaveWrite( spi, writeValue, SPI_NPCS2, 1 );
//#endif
/*
	if (error)
	{
		PrintError( "ERROR while writing to the DAC", error );
	}
*/
	return error;
}

//-----------------------------------------------------------------------------
void MakeLookupTable( void )
{
	long	date = gDate;

	if (gSettings.interval < MIN_PERIOD || gSettings.interval > MAX_PERIOD)
	{
		PrintError( "Interval", ERR_RANGE );
		return;
	}
	if (gSettings.gain < MIN_GAIN || gSettings.gain > MAX_GAIN)
	{
		PrintError( "Gain", ERR_RANGE );
		return;
	}

	if (gSettings.ionChannel == kTp)
	{
		Prints( " Make lookup table... " );

		gNumValues = gSettings.t0TP + gSettings.dtTP;
		gFirstValue = gSettings.t0TP;
	}
	else if (gSettings.ionChannel == kGaba_alpha)
	{
		gNumValues = (long)((gSettings.timeEnd - gT0[0]) / gSettings.interval + 0.5);
		gFirstValue = (long)(gT0[0] / gSettings.interval + 0.5);

		Prints( " Make lookup table... " );

		MakeGabaAlphaLT( gT0,
						 gSettings.numT0,
						 gNumValues,
						 gSettings.interval,
						 gSettings.gain,
						 gSettings.gMax,
						 gSettings.tPeak );
	}
	else if (gSettings.ionChannel == kGaba_2exp)
	{
		gNumValues = (long)((gSettings.timeEnd - gT0[0]) / gSettings.interval + 0.5);
		gFirstValue = (long)(gT0[0] / gSettings.interval + 0.5);

		Prints( " Make lookup table... " );

		MakeGaba2ExpLT( gT0,
						gSettings.numT0,
						gNumValues,
						gSettings.interval,
						gSettings.gain,
						gSettings.gPeak,
						gSettings.tauR,
						gSettings.tauD );
	}
	else if (gSettings.ionChannel == kAmpa)
	{
		if (gSettings.sd < MIN_SD || gSettings.sd > MAX_SD)
		{
			PrintError( "sd", ERR_RANGE );
			return;
		}
		if (gSettings.nSyn < MIN_NSYN || gSettings.nSyn > MAX_NSYN)
		{
			PrintError( "nsyn", ERR_RANGE );
			return;
		}

		gNumValues = (long)((gSettings.timeEnd - gT0[0]) / gSettings.interval + 0.5);
		gFirstValue = (long)(gT0[0] / gSettings.interval + 0.5);

		Prints( " Make lookup table... " );

		MakeAmpaLT( gT0,
					gSettings.numT0,
					gNumValues,
					gSettings.interval,
					gSettings.gain,
					gSettings.nSyn,
					gSettings.gMax,
					gSettings.sd,
					gSettings.tauR,
					gSettings.tauD );
	}

	else if (gSettings.ionChannel == kDynamic)
	{
		gNumValues = (long)((gSettings.timeEnd - gT0[0]) / gSettings.interval + 0.5);
		gFirstValue = (long)(gT0[0] / gSettings.interval + 0.5);

		Prints( " Make lookup table... " );

		MakeDynamicAlphaLT( gT0,
							gSettings.numT0,
							gNumValues,
							gSettings.interval,
							gSettings.gain,
							gSettings.numRS,
							gSettings.pr,
							gSettings.sigma,
							gSettings.taurec,
							gSettings.tauf,
							gSettings.gUnit,
							gSettings.tPeak );
	}

	gMltFlag = 0;
	date = gDate - date;
	Printf( " %ld s used\n", date );
}


void MakeTimes( double t0, double dt0, long number, unsigned char method )
{
	double		t1, t2;
	int			i;
	long		idum = (Get_sys_count()) & 0x7FFFFFFF;

	if (method == T_REGULAR || method == T_POISSON)
	{
		gSettings.t0 = t0;
		gSettings.dt0 = dt0;
		gSettings.numT0 = number;
		if (gT0 == NULL)
		{
			gT0 = malloc( ((gSettings.numT0) * sizeof( double )) );
		}
		else
		{
			gT0 = realloc( gT0, ((gSettings.numT0) * sizeof( double )) );
		}
		gT0[0] = t0;

		if (method == T_REGULAR)
		{
			for (i = 1; i < gSettings.numT0; i++)
			{
				gT0[i] = gT0[i-1] + dt0;
			}
		}
		else if (method == T_POISSON)
		{
			for (i = 1; i < gSettings.numT0; i++)
			{
				do
				{
					do
					{
						t2 = ran1( &idum );
					} while (t2 == 0.0);
					t1 = -log(t2) * dt0;
				} while (t1 < 0.002);
				gT0[i] = gT0[i-1] + t1;
			}
		}
	}

	gSettings.timeEnd = gT0[gSettings.numT0 - 1] + 0.1;
	gDGMax = realloc( gDGMax, ((gSettings.numT0) * sizeof( double )) );
	for (i = 0; i < gSettings.numT0; i++) gDGMax[i] = 0.0;

	gNumValues = (long)((gSettings.timeEnd - gT0[0]) / gSettings.interval + 0.5);
	gFirstValue = (long)(gT0[0] / gSettings.interval + 0.5);
	gNumTimeSteps = (long)(gSettings.timeEnd / gSettings.interval + 0.5);
	gFirstFreeRam = (gNumValues + 1UL) * sizeof( double );
	clipTimePtr	= 	(void *) (LOOKUP_TABLE + gFirstFreeRam);
	gClipCounter = 0UL;
	gTriggerCounter = 0UL;
}

//-----------------------------------------------------------------------------
/*! \brief Minimalist file editor to append char to a file.
 *
 * \note Hit ^C to exit and save file.
 */
static void AppendToFile( void )
{
	char	c;

	Prints( "\n  Simple text editor, enter char to append, ^c to exit and save\n" );

//	FlushReceiverBuffer();

	// Wait for ^q to quit.
	while (TRUE)
	{
		// If something new in the USART
		c = GetChar();
		// if this is the quit char
		if (c == CTRL_C)
		{
			// Exit the append function.
			break;
		}

		// Echo the char.
		PutChar( c );
		// Add it to the file.
		file_putc(c);
		// if it is a carriage return.
		if (c == '\r')
		{
			// Echo line feed.
			PutChar( '\n' );
			// Add line feed to the file.
			file_putc( '\n' );
		}
	}
}

/*
//-----------------------------------------------------------------------------
int StringCat( char **infoPtr, char *string, int *listLen )
{
	int			slen, len;
	char		*ptr;

	ptr = *infoPtr;
	len = *listLen;

	slen = strlen( string );
	len += slen;
	ptr = (char*)realloc( ptr, len + 1 );
	if (ptr == NULL) return NO_MEMORY;
	strcat( ptr, string );

	*listLen = len;
	*infoPtr = ptr;

	return SUCCESS;
}
*/

//-----------------------------------------------------------------------------
/*! \brief Append a data set to a file.
 *
 */
void AppendDataToFile( void )
{
	char		string[30], string1[30], string2[60];
	int			i;
	double		dValue;


	// Try to to set navigator on arg1 file.
	if (!nav_setcwd((FS_STRING)gDataFile, TRUE, TRUE)) return;

	// File exists, open it in append mode
	file_open(FOPEN_MODE_APPEND);

	GetDateString( gDate, string2 );
	file_write_buf( (U8 _MEM_TYPE_SLOW_*)string2, (U16)strlen( string2 ));
//	file_write_buf( (unsigned char *)string2, (U16)strlen( string2 ));

	sprintf( string2, "gain = %s\n", gainStrings[gSettings.gain] );
	file_write_buf( (U8 _MEM_TYPE_SLOW_*)string2, (U16)strlen( string2 ));
	Dbl2Str( string, gSettings.interval );
	dValue = 1.0 / gSettings.interval;
	Dbl2Str( string1, dValue );
	sprintf( string2, "interval = %ss (%sHz)\n", string, string1 );
	file_write_buf( (U8 _MEM_TYPE_SLOW_*)string2, (U16)strlen( string2 ));
	Dbl2Str( string, gSettings.timeEnd );
	sprintf( string2, "max. sample time = %ss\n", string );
	file_write_buf( (U8 _MEM_TYPE_SLOW_*)string2, (U16)strlen( string2 ));
	sprintf( string2, "ion channel = %s\n", ionChannels[gSettings.ionChannel] );
	file_write_buf( (U8 _MEM_TYPE_SLOW_*)string2, (U16)strlen( string2 ));

	if (gSettings.ionChannel == kGaba_alpha)
	{
		Dbl2Str( string, gSettings.gMax );
		dValue = 1.0 / gSettings.gMax;
		Dbl2Str( string1, dValue );
		sprintf( string2, "Gmax = %sS (%sOhm)\n", string, string1 );
		file_write_buf( (U8 _MEM_TYPE_SLOW_*)string2, (U16)strlen( string2 ));
		Dbl2Str( string, gSettings.tPeak );
		sprintf( string2, "tpeak = %ss\n", string );
		file_write_buf( (U8 _MEM_TYPE_SLOW_*)string2, (U16)strlen( string2 ));
	}
	else if (gSettings.ionChannel == kGaba_2exp)
	{
		Dbl2Str( string, gSettings.gPeak );
		dValue = 1.0 / gSettings.gPeak;
		Dbl2Str( string1, dValue );
		sprintf( string2, "Gpeak = %sS (%sOhm)\n", string, string1 );
		file_write_buf( (U8 _MEM_TYPE_SLOW_*)string2, (U16)strlen( string2 ));
		Dbl2Str( string, gSettings.tauR );
		sprintf( string2, "tau rising = %ss\n", string );
		file_write_buf( (U8 _MEM_TYPE_SLOW_*)string2, (U16)strlen( string2 ));
		Dbl2Str( string, gSettings.tauD );
		sprintf( string2, "tau decay = %ss\n", string );
		file_write_buf( (U8 _MEM_TYPE_SLOW_*)string2, (U16)strlen( string2 ));
	}

	else if (gSettings.ionChannel == kDynamic)
	{
		for (i = 0; i < gSettings.numT0; i++)
		{
			Dbl2Str( string, gDGMax[i] );
			sprintf( string2, "Gmax[%d] = %sS\n", i, string );
			file_write_buf( (U8 _MEM_TYPE_SLOW_*)string2, (U16)strlen( string2 ));
		}
		sprintf( string2, "n = %ld\n", gSettings.numRS );
		file_write_buf( (U8 _MEM_TYPE_SLOW_*)string2, (U16)strlen( string2 ));
//		Dbl2Str( string, gSettings.pr );
//		sprintf( string2, "prob = %s\n", string );
		sprintf( string2, "prob = %g\n", gSettings.pr );
		file_write_buf( (U8 _MEM_TYPE_SLOW_*)string2, (U16)strlen( string2 ));
//		Dbl2Str( string, gSettings.sigma);
//		sprintf( string2, "sigma = %s\n", string );
		sprintf( string2, "sigma = %g\n", gSettings.sigma );
		file_write_buf( (U8 _MEM_TYPE_SLOW_*)string2, (U16)strlen( string2 ));
		Dbl2Str( string, gSettings.taurec );
		sprintf( string2, "tau_r = %ss\n", string );
		file_write_buf( (U8 _MEM_TYPE_SLOW_*)string2, (U16)strlen( string2 ));
		Dbl2Str( string, gSettings.gUnit );
		sprintf( string2, "g_unit = %s\n", string );
		file_write_buf( (U8 _MEM_TYPE_SLOW_*)string2, (U16)strlen( string2 ));
		Dbl2Str( string, gSettings.tPeak );
		sprintf( string2, "tpeak = %ss\n", string );
		file_write_buf( (U8 _MEM_TYPE_SLOW_*)string2, (U16)strlen( string2 ));
	}
	else if (gSettings.ionChannel == kAmpa)
	{
		Dbl2Str( string, gSettings.gMax );
		dValue = 1.0 / gSettings.gMax;
		Dbl2Str( string1, dValue );
		sprintf( string2, "Gmax = %sS (%sOhm)\n", string, string1 );
		file_write_buf( (U8 _MEM_TYPE_SLOW_*)string2, (U16)strlen( string2 ));
		Dbl2Str( string, gSettings.sd);
		dValue = 1.0 / gSettings.sd;
		Dbl2Str( string1, dValue );
		sprintf( string2, "sd = %sS (%sOhm)\n", string, string1 );
		file_write_buf( (U8 _MEM_TYPE_SLOW_*)string2, (U16)strlen( string2 ));
		Dbl2Str( string, gSettings.tauR );
		sprintf( string2, "tau rising = %ss\n", string );
		file_write_buf( (U8 _MEM_TYPE_SLOW_*)string2, (U16)strlen( string2 ));
		Dbl2Str( string, gSettings.tauD );
		sprintf( string2, "tau decay = %ss\n", string );
		file_write_buf( (U8 _MEM_TYPE_SLOW_*)string2, (U16)strlen( string2 ));
		sprintf( string2, "number of synapses = %lu\n", gSettings.nSyn );
		file_write_buf( (U8 _MEM_TYPE_SLOW_*)string2, (U16)strlen( string2 ));
	}

	else if (gSettings.ionChannel == kTp)
	{
		Dbl2Str( string, gSettings.gTP );
		sprintf( string2, "Gtp = %sS\n", string );
		file_write_buf( (U8 _MEM_TYPE_SLOW_*)string2, (U16)strlen( string2 ));
		dValue = (double)gSettings.dtTP * gSettings.interval;
		Dbl2Str( string, dValue );
		sprintf( string2, "dttp = %ss\n", string );
		file_write_buf( (U8 _MEM_TYPE_SLOW_*)string2, (U16)strlen( string2 ));
		Dbl2Str( string, gSettings.eTP );
		sprintf( string2, "Etp = %sV\n", string );
		file_write_buf( (U8 _MEM_TYPE_SLOW_*)string2, (U16)strlen( string2 ));
		dValue = (double)gSettings.t0TP * gSettings.interval;
		Dbl2Str( string, dValue );
		sprintf( string2, "t0tp = %ss\n", string );
		file_write_buf( (U8 _MEM_TYPE_SLOW_*)string2, (U16)strlen( string2 ));
	}

	if (gSettings.ionChannel != kTp)
	{
		Dbl2Str( string, gSettings.eRev );
		sprintf( string2, "reversal potential = %sV\n\n", string );
		file_write_buf( (U8 _MEM_TYPE_SLOW_*)string2, (U16)strlen( string2 ));
	}

	// print out the times
	if  (
			gSettings.ionChannel == kGaba_alpha ||
			gSettings.ionChannel == kGaba_2exp	||
			gSettings.ionChannel == kDynamic
		)
	{
		sprintf( string2, "time method = %s\n", timeMethods[gSettings.tMethode] );
		file_write_buf( (U8 _MEM_TYPE_SLOW_*)string2, (U16)strlen( string2 ));
		sprintf( string2, "number of times = %lu\n", gSettings.numT0 );
		file_write_buf( (U8 _MEM_TYPE_SLOW_*)string2, (U16)strlen( string2 ));

		if (gSettings.tMethode == T_REGULAR)
		{
			Dbl2Str( string, gSettings.t0 );
			sprintf( string2, "first time = %ss\n", string );
			file_write_buf( (U8 _MEM_TYPE_SLOW_*)string2, (U16)strlen( string2 ));
			Dbl2Str( string, 1.0 / gSettings.dt0 );
			sprintf( string2, "frequency = %sHz\n", string );
			file_write_buf( (U8 _MEM_TYPE_SLOW_*)string2, (U16)strlen( string2 ));
		}
		else if (gSettings.tMethode == T_POISSON || gSettings.tMethode == T_MANUAL)
		{
			for (i = 0; i < gSettings.numT0; i++)
			{
				Dbl2Str( string, gT0[i] );
				sprintf( string2, "t0[%d] = %ss\n", i, string );
				file_write_buf( (U8 _MEM_TYPE_SLOW_*)string2, (U16)strlen( string2 ));
			}
		}
		strcpy( string2, "\n" );
		file_write_buf( (U8 _MEM_TYPE_SLOW_*)string2, (U16)strlen( string2 ));
	}

	// Close the file
	file_close();
}



//-----------------------------------------------------------------------------
/*! \brief Mount a file system.
 *
 */
static int Mount( U8 drive )
{
	int		error = 0;

	// If drive doesn't exist
	if (drive >= nav_drive_nb())
	{
		error = ERR_DRIVE;
	}
	else
	{
		// Reset all navigators.
		nav_reset();
		// Select the desired drive.
		nav_drive_set(drive);
		// Try to mount it.
		if (!nav_partition_mount())
		{
			// Set error
			error = ERR_MOUNT;
			// Retry to mount at next "ls".
			first_ls = TRUE;
		}
		else
		{
			// Clear flag, no need to remount at next "ls".
			first_ls = FALSE;
		}
	}
	return error;
}

//-----------------------------------------------------------------------------
/*! \brief Format a file system.
 *
 */
static int Format( U8 drive, U8 fatType )
{
	U8	j;
	int	error = 0;


	// if drive number isn't valid
	if (drive >= nav_drive_nb())
	{
		// Display error message.
//		PrintError( "", ERR_DRIVE );
		error = ERR_DRIVE;
	}
	else
	{
		// Get the current drive in the navigator.
		j = nav_drive_get();
		// Select drive to format.
		nav_drive_set(drive);
		// If format fails.
		if (!nav_drive_format(fatType))
		{
			// Display error message.
//			PrintError( "", ERR_FORMAT );
			error = ERR_FORMAT;
			// Return to the previous.
			nav_drive_set(j);
		}
		// Format succeds, if drives is the one we were navigating on
		else if (drive == j)
		{
			// Reset the navigators.
			nav_reset();
			// Set current drive.
			nav_drive_set(j);
			// If partition mounting fails.
			if (!nav_partition_mount())
			{
				// Display error message.
//				PrintError( "", ERR_MOUNT );
				error = ERR_MOUNT;
				// this will be the first "ls"
				first_ls = TRUE;
			}
			else
			{
				// this won't be the first "ls", system is already mounted
				first_ls = FALSE;
			}
		}
		// Format succeds, restore previous navigator drive.
		else
		{
			nav_drive_set(j);
		}
	}
	return error;
}

//-----------------------------------------------------------------------------
int RemoveFile( char *filename )
{
	Fs_index	sav_index;
	int			error = 0;

	// Save current nav position.
	sav_index = nav_getindex();
	// Try to to set navigator on arg1 folder or file.
	if (!nav_setcwd((FS_STRING)filename, TRUE, FALSE))
	{
		error = ERR_UNKNOWN_FILE;
	}
	// Try to delete the file
	else if (!nav_file_del(FALSE))
	{
		error = ERR_RM;
	}
	// Restore nav position.
	nav_gotoindex(&sav_index);

	return error;
}


//-----------------------------------------------------------------------------
long GetNumber( char *numStr, int pos, int num, int min, int max )
{
	int		i, j;
	size_t	len;
	long	value;
	char	str[11];

	len = strlen( numStr );
	if (pos >= len) return -1L;
	for (j = 0, i = pos; i < pos + num; i++, j++)
	{
		if (!isdigit( numStr[i] )) break;
		str[j] = numStr[i];
		if (numStr[i] == '\0') break;
	}
	str[j] = '\0';
	value = strtol( str, NULL, 10 );
	if (value < min || value > max) return -2L;
	return value;
}


//-----------------------------------------------------------------------------
time_t ParseDate( char *timeStr )
{
	struct tm	*tmnow;
	time_t		time;
	char		*ptr;
	long		value;


	if (timeStr == NULL || *timeStr == '\0') return 0;

	tmnow = localtime( &gDate );

	value = GetNumber( timeStr, 0, 2, 1, 12 );
	if (value < 0) return 0;
	tmnow->tm_mon = value - 1;
	value = GetNumber( timeStr, 2, 2, 1, 31 );
	if (value < 0) return 0;
	tmnow->tm_mday = value;
	value = GetNumber( timeStr, 4, 2, 0, 23 );
	if (value < 0) return 0;
	tmnow->tm_hour = value;
	value = GetNumber( timeStr, 6, 2, 0, 59 );
	if (value < 0) return 0;
	tmnow->tm_min = value;
	value = GetNumber( timeStr, 8, 4, 0, 2037 );
	if (value == (-2L)) return 0;
	if (value >= 0)
	{
		tmnow->tm_year = value;
		if (tmnow->tm_year > 1900) tmnow->tm_year -= 1900;
	}

	tmnow->tm_sec = 0;
	ptr = strchr( timeStr, '.' );
	if (ptr)
	{
		value = GetNumber( ptr, 1, 2, 0, 59 );
		if (value < 0) return 0;
		tmnow->tm_sec = value;
	}

	if((time = mktime( tmnow )) == -1) return 0;

	return time;
}


//-----------------------------------------------------------------------------
void GetDateString( time_t date, char *string )
{
	struct tm	*tmnow;
//	char *wday[] = { "Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat" };
//	char *month[] = { "Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec" };

	tmnow = localtime( &date );

	// example:	Tue May 18 12:00:16 2010

	strftime( string, 80, "%c\n", tmnow );

//	sprintf( string, "%s %s %d %02d:%02d:%02d %d\n",
//			wday[tmnow->tm_wday], month[tmnow->tm_mon], tmnow->tm_mday, tmnow->tm_hour,
//				tmnow->tm_min, tmnow->tm_sec, tmnow->tm_year + 1900 );
}

//-----------------------------------------------------------------------------
void Flip32Bit( char *adr )
{
	char c;

	if (adr)
	{
		c = adr [0];
		adr [0] = adr [3];
		adr [3] = c;

		c = adr [1];
		adr [1] = adr [2];
		adr [2] = c;
	}
}

/*
//-----------------------------------------------------------------------------
time_t GetDate()
{
#ifdef DBL
	time_t	date = 0L;
	twi_package_t			twiPacket;

	// TWI chip address to communicate with
	twiPacket.chip = RTC_ADDRESS;
	// TWI address/commands to issue to the other chip (node)
	twiPacket.addr = RTC_ADDR_START;
	// Length of the TWI data address segment (1-3 bytes)
	twiPacket.addr_length = RTC_ADDR_LGT;
	// Where to find the data to be written
	twiPacket.buffer = (void*)&date;
	// How many bytes do we want to write
	twiPacket.length = sizeof( date );

	// perform a read access
	status = twi_master_read( &AVR32_TWI, &twiPacket );

	// check read result
	if (status)
	{
		// display test result to user
		PrintError( "", (status * (-1) + TWI_NO_ERROR) );
	}

	Flip32Bit( (void*)&date );
	return date;
#else
	return gDate;
#endif
}
*/


/*
//-----------------------------------------------------------------------------
int ParseCommand( char *cmd, char *sw, unsigned long *swtest, int argc, double *argd, char *str1, char *str2 )
{
	int				error;
	char			cntc;
	unsigned long	test = 0;
	char			*ptr, *restPtr;
	double			dValue;
//	char			string[32];

	ptr = strtok( NULL, " \t\r\n" );
	if (sw[0])
	{
		error = GetOptions( cmd, sw, &cntc, &test );
		if (error == 0)
		{
			ptr = strtok( NULL, " \t\r\n" );
		}
		error = 0;
	}

	dValue = strtod( ptr, &restPtr );
	error = Str2Dbl( restPtr, &dValue );
	if (error)
	{
		error = NO_NUMBER;
		return error;
	}

	return error;
}
*/
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
int main( void )
{
//	volatile avr32_pm_t				*pm			= &AVR32_PM;
//	volatile U32 					*sdram		= (void *) BOARD_SDRAM_BASE;
//	volatile U32 					*rpFlag		= (void *) 0x24007000;
	volatile double					*sdramDPtr	= (void *) LOOKUP_TABLE;
	volatile avr32_pio_t			*pioa		= &AVR32_PIOA;
	volatile avr32_pio_t			*piob		= &AVR32_PIOB;
	volatile avr32_pio_t			*pioc		= &AVR32_PIOC;
	volatile avr32_tc_t				*tc			= &AVR32_TC0;
//	volatile avr32_wdt_t			*wdt		= &AVR32_WDT;

	static const pm_pll_opt_t		pllOpt =
	{

	  .pll_id			= 0,			// the pll id number
	  .osc_id			= 0,			// the oscillator id number to use as a reference
	  .mul				= _MULTIPLIER,	// multiplier
	  .div				= 2,			// divider
	  .count			= 16,			// the number of cycles after pll lock before status register is updated
	  .options			= 4,			// extended pll options. Consult the part specific datasheet for settings
	  .wait_for_lock	= TRUE			// set to true to wait actively for pll lock
	};

	static const pm_gen_clk_opt_t	genClk =
	{
	  .divider = 1,						// the divider from the clock source
	  .clock_source = PM_PLL0			// which clock source to use: \ref pm_clock_source_e
	};

	gpio_map_t spi_piomap =
	{
		{ AVR32_SPI0_SCK_0_PIN,		AVR32_SPI0_SCK_0_FUNCTION	},
		{ AVR32_SPI0_MISO_0_PIN,	AVR32_SPI0_MISO_0_FUNCTION	},
		{ AVR32_SPI0_MOSI_0_PIN,	AVR32_SPI0_MOSI_0_FUNCTION	},
		{ AVR32_SPI0_NPCS_0_PIN,	AVR32_SPI0_NPCS_0_FUNCTION	},
		{ AVR32_SPI0_NPCS_1_PIN,	AVR32_SPI0_NPCS_1_FUNCTION	},
		{ AVR32_SPI0_NPCS_2_PIN,	AVR32_SPI0_NPCS_2_FUNCTION	},
		{ AVR32_SPI0_NPCS_3_PIN,	AVR32_SPI0_NPCS_3_FUNCTION	}
	};

#ifdef DBL
	static const gpio_map_t twi_piomap =
	{
		{ AVR32_TWI_SDA_0_PIN,		AVR32_TWI_SDA_0_FUNCTION	},
		{ AVR32_TWI_SCL_0_PIN,		AVR32_TWI_SCL_0_FUNCTION	}
	};

	twi_options_t			twiOpt;
	twi_package_t			twiPacket;
	int						status;
#endif

	BSP_SMC_HDR				config;
	int						/*freq, */pllFreq;
	int						error, cmd, result;
	int						iValue = 0;
	unsigned long			sdramSize, i, tmp, test = 0/*, noErrors*/;
	long					val1, val2, val3;
	unsigned long			uval[4];
	double					dValue;
	double					voltage;
	char					*cmdBuffer, *ptr, *restPtr;
	char					argc = 0, tmpc;
	char					tmpUnit, string[40], string1[30];
	unsigned short			writeValue, readValue;
    double                  temp;
	double					vi[3];

	U8						k;
	U16						file_size;
	Fs_index				sav_index;
	static Fs_index			mark_index;
	const char				*part_type;
	time_t					date;

//	unsigned long			monitorLen;
	void					*sdramBase;

	unsigned long			smc[14];


	pm_reset();

	/* Disable all interrupts on PIO */
	pioa->idr = 0xFFFFFFFF;
	piob->idr = 0xFFFFFFFF;
	pioc->idr = 0xFFFFFFFF;

	config.NCSReadSetup		=   0;			// Delay from address valid to assertion of given strobe.
	config.NRDSetup			=  13;
	config.NCSWriteSetup	=   0;
	config.NWESetup			=  26;
	config.NCSReadPulse		= 146;			// Pulse length of given strobe.
	config.NRDPulse			= 133;
	config.NCSWritePulse	= 120;
	config.NWEPulse			=  80;
	config.ReadCycle		= 160;			// Total cycle length of given operation.
	config.WriteCycle		= 173;
	config.BusWidth			=   2;			// Bus width in bytes.
	config.NRDControlled	=   1;			// Data is sampled on rising edge of {0}:NCS {1}:NRD.
	config.NWEControlled	=   1;			// Data is driven on falling edge of {0}:NCS {1}:NWR.
	config.ByteWrite		=   1;			// Byte {0}:select {1}:write access type.
	config.TDFOptimization	=   0;
	config.TDFCycles		=   3;

	// Pre-config SMC CS0 for higher freq (HSB == MAIN-CLK/2)
	// needs to be run before we increase the clock frequency
	SMC_PreConfig( (void*)0, &config, (pllOpt.mul * FOSC0 / pllOpt.div) / 2, smc );

	// set up the frequencies
//	freq = pm_get_clock_source_freq_hz( PM_OSC0 );
	pllFreq = pm_start_pll( (pm_pll_opt_t*)&pllOpt );
	pm_set_clock_domain_scaler( PM_PBB_DOMAIN, 2 );
	pm_set_clock_domain_scaler( PM_PBA_DOMAIN, 4 );
	pm_set_clock_domain_scaler( PM_HSB_DOMAIN, 2 );
	pm_start_generic_clock( 0, (pm_gen_clk_opt_t*)&genClk );
	pm_set_mclk_source( PM_PLL0 );


	InitVariables();

	// use an initial setting for the timer
//	gSettings.interval = 0.00002;

	// Disable all interrupts.
	Disable_global_interrupt();

	// Initialize interrupt vectors.
	INTC_init_interrupts();

	// Register the USART interrupt handler to the interrupt controller.
	SetupUART();

	// Register the TC interrupt handler to the interrupt controller.
	INTC_register_interrupt( &TimerIntHandler, AVR32_TC0_IRQ0, AVR32_INTC_INT0 );

	// Register the PIOA interrupt handler to the interrupt controller.
	INTC_register_interrupt( &PIOAIntHandler, AVR32_PIOA_IRQ, AVR32_INTC_INT0 );

	// Enable all interrupts.
	Enable_global_interrupt();

	// Init the parallel Port
	InitPIO();

	// SPI0 gpio pins configuration
	gpio_enable_module( spi_piomap, sizeof(spi_piomap) / sizeof(spi_piomap[0]) );

	// init SPI0
	error = InitSpiMaster( spi, pm_read_module_freq_hz( PM_PBA_SPI0 ) );
	if (error)
	{
		PrintError( "ERROR while init SPI", error );
		Prints( " Please restart the board" );
	}

/*

	tmp = *rpFlag;
	if (tmp == 0x55555555)
	{
		tmp = *(rpFlag + 1);
		if (tmp == 0xAAAAAAAA)
		{
			Prints( "\n\nthe new firmware is successfully written to the flash\n" );
			Prints( "to restart, switch the device off and on again now\n" );
			*rpFlag = 0;
			while (1);
		}
	}
*/
	Prints( "\n\n" );
	PrintIdentifier();

//	Printf( "mode = 0x%08X; cycle = 0x%08X; pulse = 0x%08X; setup = 0x%08X;\n", smc[0], smc[1], smc[2], smc[3] );

#ifdef DBL
	// TWI gpio pins configuration
	gpio_enable_module( twi_piomap, sizeof(twi_piomap) / sizeof(twi_piomap[0]) );

	// options settings
	twiOpt.pba_hz = FOSC0;
	twiOpt.speed = RTC_SPEED;
	twiOpt.chip = RTC_ADDRESS;

	// initialize TWI driver with options
	status = twi_master_init( &AVR32_TWI, &twiOpt );
	// check init result
//	Printf( " status = %d; status * -1 = %d; TWI_NO_ERROR = %d; error = %d\n",
//			status, (status * (-1)), TWI_NO_ERROR, (status * (-1) + TWI_NO_ERROR) );
	if (status)
	{
		// display test result to user
		PrintError( "", (status * (-1) + TWI_NO_ERROR) );
	}
	else
	{
		// writer the EOSC bit in the control register to 0

		// TWI chip address to communicate with
		twiPacket.chip = RTC_ADDRESS;
		// TWI address/commands to issue to the other chip (node)
		twiPacket.addr = 4;
		// Length of the TWI data address segment (1-3 bytes)
		twiPacket.addr_length = RTC_ADDR_LGT;
		// Where to find the data to be written
		twiPacket.buffer = (void*)&tmpc;
		// How many bytes do we want to write
		twiPacket.length = sizeof( tmpc );

		// perform a write access
		tmpc = 0;
		status = twi_master_write( &AVR32_TWI, &twiPacket );

		// check write result
		if (status)
		{
			// display test result to user
			PrintError( "", (status * (-1) + TWI_NO_ERROR) );
		}
		else
		{
			// TWI chip address to communicate with
			twiPacket.chip = RTC_ADDRESS;
			// TWI address/commands to issue to the other chip (node)
			twiPacket.addr = RTC_ADDR_START;
			// Length of the TWI data address segment (1-3 bytes)
			twiPacket.addr_length = RTC_ADDR_LGT;
			// Where to find the data to be written
			twiPacket.buffer = (void*)&date;
			// How many bytes do we want to write
			twiPacket.length = sizeof( date );

			// perform a read access
			status = twi_master_read( &AVR32_TWI, &twiPacket );

			// check read result
			if (status)
			{
				// display test result to user
				PrintError( "", (status * (-1) + TWI_NO_ERROR) );
			}

			Flip32Bit( (void*)&date );
			gDate = date;
			GetDateString( date, string );
			Prints( " " );
			Prints( string );
		}
	}

#else
	GetDateString( gDate, string );
	Prints( " " );
	Prints( string );
	Prints( " Please set the date\n" );
#endif


	// Init the serial flash
	at45dbx_init();

	// Mount the filesystem in serial flash
	if (Mount( 0 ))
	{
		// format the filesystem in serial flash if unused
		error = Format( 0, FS_FORMAT_DEFAULT );
		if (error)
		{
			PrintError( "", error );
		}
		else
		{
			Printf( " serial flash formatted\n" );
			error = Mount( 0 );
			if (error)
			{
				PrintError( "", error );
			}
			else
			{
				Printf( " serial flash mounted\n" );
			}
		}
	}

	// Init the settings
	InitSettings( 0 );

	/* Init the Timer */
	InitTimer();

//	freq = pm_read_module_freq_hz( PM_HSB_HEBI );
//	Printf( "freq = %d\n", freq );

	Prints( " Initializing SDRAM...  " );
	sdramBase = uncached(BOARD_SDRAM_BASE);
	sdramSize = sdramc_init( sdramBase, pm_read_module_freq_hz( PM_HSB_HEBI ) );
	Prints( "done\n" );

	// setup sdram_size
//	sdramSize = 1 << (SDRAM_COL_BITS + SDRAM_ROW_BITS + SDRAM_BANK_BITS + 2);
	tmp = sdramSize;

	tmpUnit = ' ';
	if (tmp > 0x400)
	{
		tmp >>= 10;
		tmpUnit = 'k';
	}

	if (tmp > 0x400)
	{
		tmp >>= 10;
		tmpUnit = 'M';
	}

/*
	Printf( " %lu%cB SDRAM available\n", tmp, tmpUnit );

	// test the chip
	noErrors = 0UL;
	for (i = 0; i < sdramSize / 4; i++)
//	for (i = 0; i < 4; i++)
	{
		sdram[i] = ~i;
//		Printf( "i = %lu; sdram[i] = %lu\n", i, sdram[i] );
	}

	for (i = 0; i < sdramSize / 4; i++)
//	for (i = 0; i < 4; i++)
	{
		if(sdram[i] != ~i)
		{
			noErrors++;
//			Printf( " SDRAM ERROR at address 0x%08X\n", &sdram[i] );
		}
//		sdram[i] = 0UL;
//		Printf( "i = %lu; sdram[i] = %lu\n", i, sdram[i] );
	}


	if (noErrors == 0)
	{
		Prints( " SDRAM test successfully completed\n" );
//		Printf( " SDRAM address 0x%08X\n", &sdram[i] );
	}
	else
	{
	    Printf( " SDRAM test completed with %d errors out of %d tests", noErrors, i );
	}
*/

//	monitorLen = _end - _stext;
//	Printf( "_start = 0x%08X; _etext = 0x%08X; _data_lma = 0x%08X\n", _start, _etext, _data_lma );
//	Printf( "_data = 0x%08X; _end = 0x%08X\n", _data, _end );

	error = InitTimeValues();
	if (error)
	{
		PrintError( "ERROR while init time values", error );
	}
/*

	else
	{
		MakeLookupTable();
	}
*/
	WriteDAC( 0, 2048 );
	WriteDAC( 1, 2048 );

	gMltFlag = 1;
	gDataFile[0] = '\0';

	SetPrompt();


	while(1)
	{
		error = 0;
		test = 0;
		argc = 0;

		if ((cmdBuffer = ReadLine( &error )))
		{
//			Printf( " %s\n", cmdBuffer );
			// first we read a command
			ptr = strtok( cmdBuffer, " \t\r\n" );
			if (ptr != NULL && error == 0)
			{
				ConvertToLower( ptr );
				for ( cmd = 0; commands[cmd] != NULL; cmd++ )
				{
					if ( ( strcmp( commands[cmd], ptr ) == 0 ) ) break;
				}

				if ( commands[cmd] == NULL )
				{
					error = ERROR_IN_COMMAND;
				}
				cmd++;

				switch (cmd)
				{
					case HELP_CMD:
						ptr = strtok( NULL, " \t\r\n" );
						if (ptr == NULL)
						{
							Help( "" );
							break;
						}

						Help( ptr );
						break;

					case START_CMD:
						if (gMFlag)
						{
							error = CALCULATION_RUNNING;
							break;
						}

						if (gDataFile[0] == '\0')
						{
							Prints( " no data file opened\n" );
							break;
						}
						ptr = strtok( NULL, " \t\r\n" );
						result = GetOptions( ptr, "tr", &argc, &test );

						if (gMltFlag) MakeLookupTable();

						// Fill the cache
						DummyInt();

						gClipCounter = 0UL;
						gTriggerCounter = 0UL;

						if (test & 2) gRunFlag = 1;
						gMFlag = 1;

						if (test & 1)
						{
							tc_software_trigger( tc, TIMER_CHANNEL );
							gStartFlag = 1;
							gTriggerFlag = 1;
							gTimeCnt = 0UL;
							gTriggerCounter++;
							Prints( " calculation started\n" );
						}
						else
						{
							tmp = pioa->isr;
							pioa->ier	=	  (1 << AVR32_PIO_P21);		// enable interupt
							Prints( " calculation started, waiting for the trigger...\n" );
						}
						break;

					case STOP_CMD:
						if (gMFlag)
						{
							gStopFlag = 1;
							Prints( " calculation stopped\n" );
						}
						else
						{
							Prints( " calculation is not started\n" );
						}
						break;

					case GAIN_CMD:
						ptr = strtok( NULL, " \t\r\n" );
						if (ptr == NULL)
						{
							Printf( " gain: %s;\n", gainStrings[gSettings.gain] );
							break;
						}

						if (gMFlag)
						{
							error = CALCULATION_RUNNING;
							break;
						}
						dValue = strtod( ptr, &restPtr );
						if (*restPtr)
						{
							error = NO_NUMBER;
							break;
						}
						if (dValue == 0.1)
						{
							gSettings.gain = 0;
						}
						else if (dValue == 1.0)
						{
							gSettings.gain = 1;
						}
						else if (dValue == 10.0)
						{
							gSettings.gain = 2;
						}
						else if (dValue == 100.0)
						{
							gSettings.gain = 3;
						}
						else
						{
							Printf( " wrong gain value %g, use 0.1, 1, 10, or 100 [pA/mV]\n", dValue );
							break;
						}
						Printf( " gain set to %s;\n", gainStrings[gSettings.gain] );
						gMltFlag = 1;
//						MakeLookupTable();
						error = SaveSettings();
						break;

					case READ_CMD:
						ptr = strtok( NULL, " \t\r\n" );
						if (ptr == NULL) break;
						result = GetOptions( ptr, "rwd", &argc, &test );
						if (result == 0)
						{
							ptr = strtok( NULL, " \t\r\n" );
						}
						if (ptr == NULL)
						{
							error = NO_NUMBER;
							break;
						}
						iValue = strtol( ptr, NULL, 10 );
						if (test & 4)
						{
							Printf( " value[%d] = %g;\n", iValue, sdramDPtr[iValue] );
						}
						break;

					case ECHO_CMD:
						ptr = strtok( NULL, " \t\r\n" );
						if (ptr == NULL)
						{
							iValue = 1;
						}
						else
						{
							ConvertToLower( ptr );
							for ( iValue = 0; switches[iValue] != NULL; iValue++ )
							{
								if ( ( strcmp( switches[iValue], ptr ) == 0 ) ) break;
							}

							if ( switches[iValue] == NULL )
							{
								error = ERROR_IN_COMMAND;
								break;
							}
						}

						gEcho = iValue;
						Printf( " echo set to %s\n", switches[(int)gEcho] );
						break;

					case DIAGNOSTIC_CMD:
						ptr = strtok( NULL, " \t\r\n" );
						if (ptr == NULL)
						{
							iValue = 1;
						}
						else
						{
							ConvertToLower( ptr );
							for ( iValue = 0; switches[iValue] != NULL; iValue++ )
							{
								if ( ( strcmp( switches[iValue], ptr ) == 0 ) ) break;
							}

							if ( switches[iValue] == NULL )
							{
								error = ERROR_IN_COMMAND;
								break;
							}
						}

						gDiagnosticFlag = iValue;
						Printf( " diagnostic set %s\n", switches[(int)gDiagnosticFlag] );
						break;

					case GET_CMD:
						if (gMFlag)
						{
							error = CALCULATION_RUNNING;
							break;
						}
						ptr = strtok( NULL, " \t\r\n" );
						result = GetOptions( ptr, "af", &argc, &test );
						if (result == 0)
						{
							ptr = strtok( NULL, " \t\r\n" );
						}

						val1 = 1;
						iValue = 1;
						if (ptr)
						{
							if (test & 1)
							{
								val1 = strtol( ptr, NULL, 10 );
								if (val1 < 1) val1 = 1;
								if (val1 > 100) val1 = 100;
								ptr = strtok( NULL, " \t\r\n" );
							}

							if (ptr)
							{
								iValue = strtol( ptr, NULL, 10 );
//								if (iValue < 0) iValue = 0;
//								if (iValue > 1) iValue = 1;
							}
						}

						if (iValue < 1 || iValue > 2)
						{
							error = WRONG_CHANNEL;
							break;
						}

						readValue =  ReadADC( iValue - 1 );
						if (test & 2)
						{
							Prints( " type any character to abort\n");
						}

						do
						{
							// measure all 300ms
							tmpc = gHeartbeat + 3;
							if (tmpc > 9) tmpc -= 10;
							val2 = 0;
							for (i = 0; i < val1; i++)
							{
								readValue =  ReadADC( iValue );
								val2 += readValue;
							}
							readValue = val2 / val1;
							voltage = ((double)readValue / RESOLUTION * VOLTAGE_SWING - MAX_VOLT);
							Printf( "\r chan %d, average = %d: %6.4f V [%d] (0x%04X) ", iValue, val1, voltage, readValue, readValue );
							while (gHeartbeat != tmpc);
						} while ((test & 2) && !GetBufferCnt());
						Prints( "\n" );
						FlushReceiverBuffer();
						break;

					case SET_CMD:
						if (gMFlag)
						{
							error = CALCULATION_RUNNING;
							break;
						}
						ptr = strtok( NULL, " \t\r\n" );
						result = GetOptions( ptr, "dsb", &argc, &test );
						if (result == 0)
						{
							ptr = strtok( NULL, " \t\r\n" );
						}
						if (ptr == NULL)
						{
							error = NO_NUMBER;
							break;
						}

  						if (test & 1)
						{
  							val1 = strtol( ptr, NULL, 10 );
//  							writeValue = (short)val1;
						}
  						else
  						{
  							dValue = strtod( ptr, NULL );
//							val1 = ((unsigned short)((dValue + MAX_VOLT) * (RESOLUTION / VOLTAGE_SWING)));
							val1 = (long)((dValue + MAX_VOLT) * (RESOLUTION / VOLTAGE_SWING));
						}
						if (val1 < 0) val1 = 0;
						if (val1 > 4095) val1 = 4095;
						writeValue = (unsigned short)val1;

#ifdef DBL
						readValue = 0;
						if (test & 2) readValue = 1;
						if (test & 4)	// write to both channels
						{
							WriteDAC( 0, (unsigned short)writeValue );
							WriteDAC( 1, (unsigned short)writeValue );
							Printf( " set value: %04d (0x%04X) to channel 0 and 1\n", writeValue, writeValue );
						}
						else
						{
							WriteDAC( readValue, (unsigned short)writeValue );
							Printf( " set value: %04d (0x%04X) to channel %d\n", writeValue, writeValue, readValue );
						}
#else
						WriteDAC( 0, (unsigned short)writeValue );
						Printf( " set value: %04d (0x%04X)\n", writeValue, writeValue );
#endif
						break;

					case OFFSET_CMD:
						if (gMFlag)
						{
							error = CALCULATION_RUNNING;
							break;
						}
						val1 = 0;
						val2 = 1;
						ptr = strtok( NULL, " \t\r\n" );
						if (ptr)
						{
							dValue = strtod( ptr, &restPtr );
							error = Str2Dbl( restPtr, &dValue );
							if (error)
							{
								error = ERR_RANGE;
								break;
							}
							ptr = strtok( NULL, " \t\r\n" );
							if (ptr == NULL)
							{
								voltage = dValue;
								dValue = 1.0;
							}
							else
							{
								voltage = strtod( ptr, &restPtr );
								error = Str2Dbl( restPtr, &voltage );
								if (error)
								{
									error = ERR_RANGE;
									break;
								}
							}
//							Printf( "voltage = %g\n", voltage );

							val1 = (long)dValue - 1L;
							val2 = (long)(voltage * (RESOLUTION / VOLTAGE_SWING));
//							if (val2 < 0) val2 = 0;
//							if (val2 > 4095) val2 = 4095;
							writeValue = (unsigned short)val2;
							gSettings.offset[val1] = writeValue;
							val2 = val1;
							error = SaveSettings();
						}

						for (val3 = val1; val3 <= val2; val3++)
						{
							voltage = (double)gSettings.offset[val3] / (RESOLUTION / VOLTAGE_SWING);
							Dbl2Str( string, voltage );
							Printf( " offset %ld: %sV %d (0x%04X)\n", val3 + 1L, string, gSettings.offset[val3], gSettings.offset[val3] );
						}
						break;

					case FILE_CMD:
					case OPEN_CMD:
						ptr = strtok( NULL, " \t\r\n" );
						if (ptr == NULL)
						{
							if (gDataFile[0] == '\0')
							{
								Prints( " no data file opened\n" );
							}
							else
							{
								Printf( " data file: %s\n", gDataFile );
							}
							break;
						}
						if (gMFlag)
						{
							error = CALCULATION_RUNNING;
							break;
						}

						if (nav_setcwd((FS_STRING)ptr, TRUE, FALSE))
						{
							// Display error message.
							Prints( " file exists, do you want to append/overwrite/cancel? (a/o/c)\n" );
							tmpc = GetChar();
							if (tmpc == 'c')
							{
								break;
							}
							else if (tmpc == 'o')
							{
								error = RemoveFile( ptr );
								if (error) break;
							}
							else if (tmpc == 'a')
							{
								strcpy( gDataFile, ptr );
								break;
							}
						}
						// Create the file.
						if (!nav_file_create((FS_STRING)ptr))
						{
							error = ERR_CREATE;
							break;
						}
						strcpy( gDataFile, ptr );
						Printf( " data file \"%s\" created\n", gDataFile );
						break;

					case SETTINGS_CMD:
						ptr = strtok( NULL, " \t\r\n" );
						result = GetOptions( ptr, "a", &argc, &test );

//						Printf( "\tident: %08X\n", gSettings.ident );
						Printf( "\tversion: %d\n", gSettings.version );
						Printf( "\tdata file: %s\n", gDataFile );
						Printf( "\tgain: %s\n", gainStrings[gSettings.gain] );
						Printf( "\tion channel: %s\n", ionChannels[gSettings.ionChannel] );
						Dbl2Str( string, gSettings.interval );
						dValue = 1.0 / gSettings.interval;
						Dbl2Str( string1, dValue );
						Printf( "\tinterval: %ss (%sHz)\n", string, string1 );
						Dbl2Str( string, gSettings.timeEnd );
						Printf( "\ttimeEnd: %ss\n", string );
						Dbl2Str( string, gSettings.tPeak );
						Printf( "\ttPeak: %ss\n", string );
						Dbl2Str( string, gSettings.eRev );
						Printf( "\teREv: %sV\n", string );
						Dbl2Str( string, gSettings.gMax );
						dValue = 1.0 / gSettings.gMax;
						Dbl2Str( string1, dValue );
						Printf( "\tgMax: %sS (%sOhm)\n", string, string1 );
						Dbl2Str( string, gSettings.pr );
						Printf( "\tpr: %s\n", string );
						Printf( "\tnumRS: %ld\n", gSettings.numRS );
						Dbl2Str( string, gSettings.sigma);
						Printf( "\tsigma: %s\n", string );
						Dbl2Str( string, gSettings.taurec );
						Printf( "\ttaurec: %ss\n", string );
						Dbl2Str( string, gSettings.tauf );
						Printf( "\ttauf: %ss\n", string );
						Dbl2Str( string, gSettings.gUnit );
						Printf( "\tgUnit: %s\n", string );
						Dbl2Str( string, gSettings.sd);
						Dbl2Str( string1, 1.0/gSettings.sd);
						Printf( "\tsd: %sS (%sOhm)\n", string, string1 );
						Printf( "\tnSyn: %lu\n", gSettings.nSyn );
						Dbl2Str( string, gSettings.gPeak );
						dValue = 1.0 / gSettings.gPeak;
						Dbl2Str( string1, dValue );
						Printf( "\tgPeak: %sS (%sOhm)\n", string, string1 );
						Dbl2Str( string, gSettings.tauR );
						Printf( "\ttauR: %ss\n", string );
						Dbl2Str( string, gSettings.tauD );
						Printf( "\ttauD: %ss\n", string );

						Dbl2Str( string, gSettings.gTP );
						Printf( "\tgTP: %sS\n", string );
						Dbl2Str( string, gSettings.eTP );
						Printf( "\teTP: %sV\n", string );
						dValue = (double)gSettings.dtTP * gSettings.interval;
						Dbl2Str( string, dValue );
						Printf( "\tdtTP: %ss\n", string );
						dValue = (double)gSettings.t0TP * gSettings.interval;
						Dbl2Str( string, dValue );
						Printf( "\tt0TP: %ss\n", string );

						Dbl2Str( string, gSettings.t0 );
						Printf( "\tt0: %ss\n", string );
						Dbl2Str( string, gSettings.dt0 );
						Printf( "\tdt0: %ss\n", string );
						Printf( "\tnumT0: %d\n", gSettings.numT0 );
						Printf( "\ttMethod: %s\n", timeMethods[gSettings.tMethode] );
						Printf( "\tdate: %ld\n", gSettings.date );
						Printf( "\toffset 1: %d\n", gSettings.offset[0] );
						Printf( "\toffset 2: %d\n", gSettings.offset[1] );

						if (test)
						{
							Printf( "\n\tgNumValues: %ld\n", gNumValues );
							Printf( "\tgFirstValue: %ld\n", gFirstValue );
							Printf( "\tgNumTimeSteps: %ld\n", gNumTimeSteps );
							Printf( "\tgFirstFreeRam: %ld\n\n", gFirstFreeRam );

	//						gTimerValue = (pm_read_module_freq_hz( PM_PBB_TIMER0 ) / 1000000 / 		// value for period
	//								(1 << (TIMER_CLK_SRC + 1)) * (long)(gSettings.interval * 1000000.0));
							Printf( "\ttimerFreq = %d\n", pm_read_module_freq_hz( PM_PBB_TIMER0 ) );
							Printf( "\tclockSrc = %d\n", 1 << (TIMER_CLK_SRC + 1) );
	//						Printf( "\tinterval = %ld\n", (long)(gSettings.interval * 1000000.0) );
							Printf( "\tgTimerValue = %ld\n", gTimerValue );

							Printf( "\n\tg1msCounter = %lu\n", g1msCounter );
							Printf( "\tg100msCounter = %lu\n", g100msCounter );
							Printf( "\tgMilliSeconds = %lu\n", gMilliSeconds );
							Printf( "\tgVal1ms = %lu\n", gVal1ms );
							Printf( "\tgHeartbeat = %ld\n", gHeartbeat );
							Printf( "\tgTimeCnt = %lu\n", gTimeCnt );
							Printf( "\tgMFlag = %ld\n", gMFlag );
							Printf( "\tgStartFlag = %ld\n", gStartFlag );
							Printf( "\tgStopFlag = %ld\n", gStopFlag );
							Printf( "\tgTriggerFlag = %ld\n", gTriggerFlag );
							Printf( "\tgRunFlag = %ld\n", gRunFlag );
							Printf( "\tgPauseFlag = %ld\n", gPauseFlag );
							Printf( "\tgFirstFlag = %ld\n", gFirstFlag );
							Printf( "\tgMltFlag = %ld\n", gMltFlag );
	//						Printf( "\tgEcho = %ld\n", gEcho );
							Printf( "\tfirmwareFlag = %ld\n", firmwareFlag );
							Printf( "\tgClip = %ld\n", gClip );
							Printf( "\tgClipCounter = %ld\n", gClipCounter );
							Printf( "\tgLastClipTime = %ld\n", gLastClipTime );
							Printf( "\tgTriggerCounter = %ld\n", gTriggerCounter );
						}
						break;


					case INFO_CMD:
//						Printf( " trigger polarity: %s\n", polarities[gTriggerPolarity] );
						Printf( " ion channel: %s\n", ionChannels[gSettings.ionChannel] );
						Printf( " gain: %s\n", gainStrings[gSettings.gain] );
						Dbl2Str( string, gSettings.interval );
						dValue = 1.0 / gSettings.interval;
						Dbl2Str( string1, dValue );
						Printf( " sampling interval: %ss (%sHz)\n", string, string1 );
						Dbl2Str( string, gSettings.timeEnd );
						Printf( " max. sample time: %ss\n", string );

						if (gSettings.ionChannel == kGaba_alpha)
						{
//							Prints( " gaba alpha channel parameters:\n" );
							Dbl2Str( string, gSettings.gMax );
							dValue = 1.0 / gSettings.gMax;
							Dbl2Str( string1, dValue );
							Printf( " Gmax: %sS (%sOhm)\n", string, string1 );
							Dbl2Str( string, gSettings.tPeak );
							Printf( " tpeak: %ss\n", string );
						}
						else if (gSettings.ionChannel == kGaba_2exp)
						{
//							Prints( " gaba 2exp channel parameters:\n" );
							Dbl2Str( string, gSettings.gPeak );
							dValue = 1.0 / gSettings.gPeak;
							Dbl2Str( string1, dValue );
							Printf( " Gpeak: %sS (%sOhm)\n", string, string1 );
							Dbl2Str( string, gSettings.tauR );
							Printf( " tau rising: %ss\n", string );
							Dbl2Str( string, gSettings.tauD );
							Printf( " tau decay: %ss\n", string );
						}
						else if (gSettings.ionChannel == kDynamic)
						{
//							Prints( " dynamic alpha channel parameters:\n" );

							for (i = 0; i < gSettings.numT0; i++)
							{
								Dbl2Str( string1, gDGMax[i] );
								Printf( " Gmax[%d] = %sS\n", i, string1 );
							}
							Printf( " n: %ld\n", gSettings.numRS );
							Dbl2Str( string, gSettings.pr );
							Printf( " pr: %s\n", string );
							Dbl2Str( string, gSettings.sigma);
							Printf( " sigma: %s\n", string );
							Dbl2Str( string, gSettings.taurec );
							Printf( " tau_r: %ss\n", string );
							Dbl2Str( string, gSettings.tauf );
							Printf( " tau_f: %ss\n", string );
							Dbl2Str( string, gSettings.gUnit );
							Printf( " g_unit: %s\n", string );
							Dbl2Str( string, gSettings.tPeak );
							Printf( " tpeak: %ss\n", string );
						}
						else if (gSettings.ionChannel == kAmpa)
						{
//							Prints( " ampa channel parameters:\n" );
							Dbl2Str( string, gSettings.gMax );
//							dValue = 1.0 / gSettings.gMax;
//							Dbl2Str( string1, dValue );
							Dbl2Str( string1, 1.0 / gSettings.gMax );
							Printf( " gmax: %sS (%sOhm)\n", string, string1 );
							Dbl2Str( string, gSettings.sd);
							Dbl2Str( string1, 1.0/gSettings.sd);
							Printf( " sd: %sS (%sOhm)\n", string, string1 );
							Dbl2Str( string, gSettings.tauR );
							Printf( " tau rising: %ss\n", string );
							Dbl2Str( string, gSettings.tauD );
							Printf( " tau decay: %ss\n", string );
//							Dbl2Str( string, gSettings.eRev );
//							Printf( " reversal potential: %sV\n", string );
							Printf( " the number of synapses: %lu\n", gSettings.nSyn );
						}

						else if (gSettings.ionChannel == kTp)
						{
//							Prints( " test pulse parameters:\n" );
							Dbl2Str( string, gSettings.gTP );
							Printf( " Gtp: %sS\n", string );
							dValue = (double)gSettings.dtTP * gSettings.interval;
							Dbl2Str( string, dValue );
							Printf( " dttp: %ss\n", string );
							Dbl2Str( string, gSettings.eTP );
							Printf( " Etp: %sV\n", string );
							dValue = (double)gSettings.t0TP * gSettings.interval;
							Dbl2Str( string, dValue );
							Printf( " t0tp: %ss\n", string );

						}

						if (gSettings.ionChannel != kTp)
						{
							Dbl2Str( string, gSettings.eRev );
							Printf( " reversal potential: %sV\n\n", string );
						}

						// print out the times
						if  (
								gSettings.ionChannel == kGaba_alpha ||
								gSettings.ionChannel == kGaba_2exp	||
								gSettings.ionChannel == kDynamic
							)
						{
							Printf( " time methode = %s\n", timeMethods[gSettings.tMethode] );
							Printf( " number of times = %lu\n", gSettings.numT0 );
							if (gSettings.tMethode == T_REGULAR)
							{
								Dbl2Str( string1, gT0[0] );
								Printf( " first time: %ss\n", string1 );
//								dValue = 1.0 / (gT0[1] - gT0[0]);
								dValue = 1.0 / gSettings.dt0;
								Dbl2Str( string1, dValue );
								Printf( " frequency: %sHz\n", string1 );
							}
							else if (gSettings.tMethode == T_POISSON || gSettings.tMethode == T_MANUAL)
							{
								for (i = 0; i < gSettings.numT0; i++)
								{
									Dbl2Str( string1, gT0[i] );
									Printf( " t0[%d] = %ss\n", i, string1 );
								}
							}
						}
//						Prints( "\n" );
						break;

					case INTERVAL_CMD:
						ptr = strtok( NULL, " \t\r\n" );
						if (ptr == NULL)
						{
							Dbl2Str( string, gSettings.interval );
							dValue = 1.0 / gSettings.interval;
							Dbl2Str( string1, dValue );
							Printf( " sampling interval: %ss (%sHz)\n", string, string1 );
							break;
						}
						if (gMFlag)
						{
							error = CALCULATION_RUNNING;
							break;
						}
						dValue = strtod( ptr, &restPtr );
						error = Str2Dbl( restPtr, &dValue );
						if (error || dValue < MIN_PERIOD)
						{
							error = ERR_RANGE;
							break;
						}
						gSettings.interval = dValue;

						CalcTimerValues();
						tc_write_rc( tc, TIMER_CHANNEL, gTimerValue );

						Dbl2Str( string, gSettings.interval );
//						Printf( " val = %g; restPtr = %p; rest = %s; gSettings.interval = %g; gSettings.interval = %s;\n",
//									dValue, restPtr, restPtr, gSettings.interval, string );
						dValue = 1.0 / gSettings.interval;
						Dbl2Str( string1, dValue );
						Printf( " sampling interval set to %ss (%sHz)\n", string, string1 );
						error = SaveSettings();
						break;

					case GDYNAMIC_CMD:
						Printf( " the number of dynamic gmax: %d\n", gSettings.numT0 );
						for (i = 0; i < gSettings.numT0; i++)
						{
							Dbl2Str( string1, gDGMax[i] );
							Printf( " Gmax[%d] = %sS\n", i, string1 );
						}
						break;

					case DATE_CMD:
						ptr = strtok( NULL, " \t\r\n" );

#ifdef DBL
						// TWI chip address to communicate with
						twiPacket.chip = RTC_ADDRESS;
						// TWI address/commands to issue to the other chip (node)
						twiPacket.addr = RTC_ADDR_START;
						// Length of the TWI data address segment (1-3 bytes)
						twiPacket.addr_length = RTC_ADDR_LGT;
						// Where to find the data to be written
						twiPacket.buffer = (void*)&date;
						// How many bytes do we want to write
						twiPacket.length = sizeof( date );
#endif

						if (ptr)
						{
							if (gMFlag)
							{
								error = CALCULATION_RUNNING;
								break;
							}

							date = ParseDate( ptr );
							if (date == 0)
							{
								error = ERROR_IN_DATE;
								break;
							}

							gDate = date;
							gMilliSeconds = 0;

#ifdef DBL
							Flip32Bit( (void*)&date );
							// perform a write access
							status = twi_master_write( &AVR32_TWI, &twiPacket );

							// check write result
							if (status)
							{
								// display write result to user
								error = status * (-1) + TWI_NO_ERROR;
								break;
							}
#endif
						}
#ifdef DBL
						// perform a read access
						status = twi_master_read( &AVR32_TWI, &twiPacket );

						// check read result
						if (status)
						{
							// display test result to user
							PrintError( "", (status * (-1) + TWI_NO_ERROR) );
						}

						Flip32Bit( (void*)&date );
//						Printf( " date = %ld (0X%08X)\n", date, date );
						gDate = date;
#endif
						GetDateString( gDate, string );
						Prints( " " );
						Prints( string );

						error = 0;
						break;

					case TIME_CMD:
						ptr = strtok( NULL, " \t\r\n" );
						if (ptr)
						{
							if (gMFlag)
							{
								error = CALCULATION_RUNNING;
								break;
							}
							result = GetOptions( ptr, "arp", &argc, &test );
							if (result == 0)
							{
								ptr = strtok( NULL, " \t\r\n" );
							}
							if (test == 0) gSettings.numT0 = 0;

							if (test == 0 || test == 1)			// manual times
							{
								gSettings.tMethode = T_MANUAL;
								while (ptr)
								{
									dValue = strtod( ptr, &restPtr );
									error = Str2Dbl( restPtr, &dValue );
									if (error)
									{
										error = NO_NUMBER;
										break;
									}
									gT0 = realloc( gT0, ((gSettings.numT0 + 1) * sizeof( double )) );
									gT0[gSettings.numT0++] = dValue;
									ptr = strtok( NULL, " \t\r\n" );
								}

//								gSettings.timeEnd = dValue + 0.1;
							}
							else if (test == 2 || test == 4)	// regular and poisson times
							{
								for (i = 0; i < 3; i++)
								{
									if (ptr == NULL)
									{
										error = NO_NUMBER;
										break;
									}
									dValue = strtod( ptr, &restPtr );
									error = Str2Dbl( restPtr, &dValue );
									if (error)
									{
										error = NO_NUMBER;
										break;
									}
									vi[i] = dValue;
									ptr = strtok( NULL, " \t\r\n" );
								}

								if (test == 2)
								{
									gSettings.tMethode = T_REGULAR;
								}
								else if (test == 4)
								{
									gSettings.tMethode = T_POISSON;
								}
							}

							MakeTimes( vi[0], 1.0 / vi[1], (long)vi[2], gSettings.tMethode );
							gMltFlag = 1;
							error = SaveSettings();
							if (error) break;
						}

						Printf( " time method: %s\n", timeMethods[gSettings.tMethode] );
						Printf( " number of times: %d\n", gSettings.numT0 );
						for (i = 0; i < gSettings.numT0; i++)
						{
							Dbl2Str( string1, gT0[i] );
							Printf( "\tt0[%d] = %ss\n", i, string1 );
//							Printf( "t0[%d] = %g\n", i, gT0[i] );
						}

						break;

					case MAKELT_CMD:
						MakeLookupTable();
						break;


					case MAXTIME_CMD:
						ptr = strtok( NULL, " \t\r\n" );
						if (ptr == NULL)
						{
							Printf( " max. running time: %g s\n", gSettings.timeEnd );
							break;
						}
						if (gMFlag)
						{
							error = CALCULATION_RUNNING;
							break;
						}
						dValue = strtod( ptr, &restPtr );
						result = Str2Dbl( restPtr, &dValue );
						if (result)
						{
							error = NO_NUMBER;
							break;
						}
						gSettings.timeEnd = dValue;

						gNumValues = (long)((gSettings.timeEnd - gT0[0]) / gSettings.interval + 0.5);
						gFirstValue = (long)(gT0[0] / gSettings.interval + 0.5);
						gNumTimeSteps = (long)(gSettings.timeEnd / gSettings.interval + 0.5);
						gFirstFreeRam = (gNumValues + 1UL) * sizeof( double );
						clipTimePtr	= 	(void *) (LOOKUP_TABLE + gFirstFreeRam);
						gClipCounter = 0UL;
						gTriggerCounter = 0UL;

						Printf( " max. sampling time set to: %g s\n", gSettings.timeEnd );
						gMltFlag = 1;
						error = SaveSettings();
						break;

					case BAUD_CMD:
						ptr = strtok( NULL, " \t\r\n" );
						if (ptr == NULL)
						{
							GetBaudRate( gSpiADSpeed, pm_read_module_freq_hz( PM_PBA_SPI0 ), &tmp, string );
							Printf( " AD/DA baudrate: %s\n", string );
							GetBaudRate( gSpiSFSpeed, pm_read_module_freq_hz( PM_PBA_SPI0 ), &tmp, string );
							Printf( " SPI SF baudrate: %s\n", string );
							break;
						}
						if (gMFlag)
						{
							error = CALCULATION_RUNNING;
							break;
						}
						result = GetOptions( ptr, "af", &argc, &test );
						if (result)
						{
							error = WRONG_OPTION;
							break;
						}
						ptr = strtok( NULL, " \t\r\n" );
						tmp = strtol( ptr, NULL, 10 );
						if (tmp == 0)
						{
							error = NO_ARGUMENT;
							break;
						}
						if (test == 1)
						{
							gSpiADSpeed = tmp;
						}
						else if (test == 2)
						{
							gSpiSFSpeed = tmp;
						}
						error = InitSpiMaster( spi, pm_read_module_freq_hz( PM_PBA_SPI0 ) );
						GetBaudRate( gSpiADSpeed, pm_read_module_freq_hz( PM_PBA_SPI0 ), &tmp, string );
						Printf( " AD/DA baudrate set to: %s\n", string );
						GetBaudRate( gSpiSFSpeed, pm_read_module_freq_hz( PM_PBA_SPI0 ), &tmp, string );
						Printf( " SPI SF baudrate set to: %s\n", string );
						break;

					case RESTART_CMD:
//						goto _start;
//						wdt_reset_mcu();
//						Disable_global_interrupt();
						_initprog();
						break;

					case DUMP_CMD:
						if (gMFlag)
						{
							error = CALCULATION_RUNNING;
							break;
						}
						ptr = strtok( NULL, " \t\r\n" );
						if (ptr == NULL)
						{
							uval[0] = 0x10000000;
						}
						else
						{
							uval[0] = strtoul( ptr, NULL, 16 );
							uval[0] &= ~3UL;
							ptr = strtok( NULL, " \t\r\n" );
						}

						if (ptr == NULL)
						{
							uval[1] = 0xFF;
						}
						else
						{
							uval[1] = strtoul( ptr, NULL, 16 );
						}
//						uval[1] = (((uval[1] - 1) / 16 + 1) * 16) - 1;
						uval[1] = (uval[1] & 0xFFFFFFF0) + 0x0F;
						Printf( "show memory from %08X to %08X\n", uval[0], uval[0] + uval[1] );

						{
							long			tmp = uval[1];
							volatile char	*ptr = (void *) uval[0];
							char			ch;

							while (tmp > 0)
							{
								Printf( "%08X ", ptr );
								for (i = 0; i < 16; i++)
								{
									if (i == 8) Prints( "  ");
									Printf( "%02X ", *(ptr + i) );
								}
								Prints( " " );
								for (i = 0; i < 16; i++)
								{
									ch = *(ptr + i);
									if (ch < 0x20 || ch > 0x7E) ch = '.';
									Printf( "%c", ch );
								}
								Prints( "\n" );

								ptr = ptr + 16;
								tmp -= 16;
							}
							Printf( "\n" );
						}

/*
						for (i = val1; i <= val2; i++)
						{
//							Printf( " value[%d] = %d;\n", i, sdram[i] );
							Printf( " value[%d] = %g;\n", i, sdramDPtr[i] );
						}
*/
						break;

					case FIRMWARE_CMD:
						ptr = strtok( NULL, " \t\r\n" );
						result = GetOptions( ptr, "f", &argc, &test );
						Prints( "ready to read the firmware file\n" );
						LoadFile();
						Prints( "new firmware loaded into RAM\n" );
						gMltFlag = 1;

					case CHECK_FW_CMD:
						firmwareFlag = 0;
						error = CheckFile();
						if (error) break;

						firmwareFlag = 1;
						if (test == 0)
						{
							Prints( "firmware successfully checked\n" );
							Prints( "it can be written to flash now, using the \"flash\" command\n" );
							break;
						}

					case WRITE_FLASH_CMD:
						if (firmwareFlag == 0)
						{
							Prints( "please load the firmware first\n" );
							break;
						}
						Prints( "the firmware will be written to the flash now\n" );
						Prints( "do not switch off the device while the flash will be programed\n" );
//						Prints( "when the prompt occures, restart the device by switching off and on again\n" );
						Disable_global_interrupt();
						clipTimePtr = NULL;
						if (gT0) free(gT0);
						gT0 = NULL;

						WriteFirmware();

						break;

					case LED_CMD:
						ptr = strtok( NULL, " \t\r\n" );
						if (ptr)
						{
							uval[0] = strtoul( ptr, NULL, 10 );
							if (uval[0])
							{
								pioa->codr	=	(1 << AVR32_PIO_P16);	// set output to low (LED on)
							}
							else
							{
								pioa->sodr	=	(1 << AVR32_PIO_P16);	// set output to low (LED on)
							}

						}
						break;

					case TEST_CMD:
/*
						ptr = strtok( NULL, " \t\r\n" );
						if (ptr) result = GetOptions( ptr, "fiep", &argc, &test );
						if (result == 0)
						{
							ptr = strtok( NULL, " \t\r\n" );
						}
						if (ptr == NULL)
						{
//							uval[0] = 0x7F0000;
							uval[0] = smc[0];
						}
						else
						{
							uval[0] = strtoul( ptr, NULL, 16 );
							ptr = strtok( NULL, " \t\r\n" );
						}
						if (ptr == NULL)
						{
							uval[1] = smc[1];
						}
						else
						{
							uval[1] = strtoul( ptr, NULL, 16 );
							ptr = strtok( NULL, " \t\r\n" );
						}
						if (ptr == NULL)
						{
							uval[2] = smc[2];
						}
						else
						{
							uval[2] = strtoul( ptr, NULL, 16 );
							ptr = strtok( NULL, " \t\r\n" );
						}
						Printf( "test = %d\n", test );
						// Disable all interrupts.
						Disable_global_interrupt();

						if (test == 1)
						{
							WriteFirmware();
						}
						else if (test == 2)
						{
							error = GetFlashID();
						}
						else if (test == 4)
						{
							error = EraseSector( 0x007F0000 );
						}
						else if (test == 8)
						{
							error = FlashWrite( 0x007F0000, 0x6677 );
						}
						else
						{
							Printf( "count = 0x%08X; address = 0x%08X; ram = 0x%08X\n",
										uval[0], uval[1], uval[2] );
							Test( smc, uval[0], uval[1], uval[2] );
							Printf( "count = 0x%08X; address = 0x%08X; ram = 0x%08X; data = 0x%08X\n",
										smc[0], smc[1], smc[2], smc[3] );
						}

						// Enable all interrupts.
						Enable_global_interrupt();

						if (test == 2)
						{
							Printf( "Flash MC = 0x%08X\n", error );
							error = 0;
						}
						else if (test == 4 || test == 8)
						{
							if (error)
							{
								Prints( "sector erase/program not successful\n" );
								error = 0;
							}
							else
							{
								Prints( "sector erase/program successful\n" );
							}
						}
*/
/*
						{
							volatile char	*ptr = (void *) BOARD_SDRAM_BASE;
							char			ch, flag = 0, cnt = 0;
							time_t			timeout;

							while ((GetBufferCnt()) == 0);			// wait for the first character

							while (1)
							{
								timeout = gDate + 2;
								while ((GetBufferCnt()) == 0)
								{
									if (gDate == timeout)
									{
										flag = 1;
										break;
									}
								}
								if (flag) break;
								ch = GetChar();
								*ptr = ch;
								ptr++;

								if (ch == 'S')
								{
									Prints( "." );
									if (cnt++ > 80)
									{
										Prints( "\n" );
										cnt = 0;
									}
								}
							}
							Prints( "\n" );
						}
*/

//						error = LoadTimes();
/*
						ptr = strtok( NULL, " \t\r\n" );
						if (ptr == NULL)
						{
							Help( "" );
							break;
						}

						Help( ptr );
*/
/*
						for (i = 0; i < gSettings.numT0; i++)
						{
							Dbl2Str( string1, gDGMax[i] );
							Printf( "Gmax[%d] = %sS\n", i+1, string1 );
						}
						break;
*/

						/*
						if (gMFlag)
						{
							error = CALCULATION_RUNNING;
							break;
						}
						ptr = strtok( NULL, " \t\r\n" );
						error = GetOptions( ptr, "isab", &argc, &test );
//						Printf( "error = %d; test = %04X\n", error, test );

						if (test == 0)
						{
						}
						else if (test == 1)
						{
//							error = SFReadDevID( id );
//							for (i = 0; i < 4; i++)
//							{
//								Printf( "flash id[%d] = %02X\n", i, id[i] );
//							}
						}
						else if (test == 2)
						{
							if (at45dbx_mem_check())
							{
								Prints( "flash status [PASS]\n" );
							}
							else
							{
								Prints( "flash status [FAIL]\n" );
							}
						}
						else if (test == 4)
						{
							tmp = sizeof( gSettings );
							Printf( "sizeof( gSettings ) = %ld\n", tmp );
							Printf( "sizeof( dcSettingsRecord ) = %ld\n", tmp );
						}
						else if (test == 8)
						{
							char c;

							while (TRUE)
							{
								// If something new in the USART
								c = GetChar();
								// if this is not the quit char
								if (c == '\r')
								{
									// Exit the append function.
									break;
								}
								Printf( "%02X ", c );
							}
						}
*/


/*
						ptr = strtok( NULL, " \t\r\n" );
						if (ptr == NULL)
						{
							spi_empty_rd( spi );
							error = spi_variableSlaveWrite( spi, 0, SPI_NPCS1, 1 );
							if (error)
							{
								PrintError( "ERROR while starting the ADC", error );
							}

							error = spi_read( spi, &readValue );
							if (error)
							{
								PrintError( "ERROR while reading from the ADC", error );
							}

							voltage = ((double)readValue / 4096.0 * 2 - 1);
							Printf( "value = %g V [%d] (0x%04X)\n", voltage, readValue, readValue );
							break;
						}
						else
						{
							val1 = strtol( ptr, NULL, 10 );

							ptr = strtok( NULL, " \t\r\n" );
							if (ptr == NULL)
							{
								val2 = 2;
							}
							else
							{
								val2 = strtol( ptr, NULL, 10 );
							}
						}

						if (val1 < 0) val1 = 0;
						if (val1 > 4095) val1 = 4095;

						writeValue = (unsigned short)val1 << val2;
						error = spi_variableSlaveWrite( spi, writeValue, SPI_NPCS2, 1 );
						if (error)
						{
							PrintError( "ERROR while writing to the DAC", error );
							break;
						}

						Printf( "set value %d (DAC set 0x%04X)\n", val1, writeValue );
*/
/*

						ptr = strtok( NULL, " \t\r\n" );

						gClipCounter = 0UL;
						gTriggerCounter = 1UL;
						gLastClipTime = 0UL;
						while (ptr)
						{
							val1 = strtol( ptr, &restPtr, 10 );
							if (*restPtr)
							{
								error = NO_NUMBER;
								break;
							}

							if ((gClipCounter == 0UL) || ((val1 - 1) > gLastClipTime))
							{
								clipTimePtr[gClipCounter].trigger = gTriggerCounter;
								clipTimePtr[gClipCounter].time = val1;
								gClipCounter++;

								Printf( "gClipCounter = %d; val1 = %ld\n", gClipCounter, val1 );

//								vPtr = (char *)&gTriggerCounter;
//								for (i = 0; i < 4; i++) aa[i] = *(char *)vPtr++;

//								vPtr = (char *)&testCnt;
//								for (i = 4; i < 12; i++) aa[i] = *(char *)vPtr++;

//								vPtr = (unsigned long *)&aa[0];
//								sdram[(gFirstFreeRam / 4) + (gClipCounter * 3)] = *(unsigned long *)vPtr;
//								vPtr = (unsigned long *)&aa[4];
//								sdram[(gFirstFreeRam / 4) + (gClipCounter * 3) + 1] = *(unsigned long *)vPtr;
//								vPtr = (unsigned long *)&aa[8];
//								sdram[(gFirstFreeRam / 4) + (gClipCounter * 3) + 2] = *(unsigned long *)vPtr;
//								gClipCounter++;
							}
							gLastClipTime = val1;
							ptr = strtok( NULL, " \t\r\n" );
						}
*/

/*
						Printf( "&clipTimePtr = %p; gClipCounter = %d\n", clipTimePtr, gClipCounter );
						for (i = 0; i < gClipCounter; i++)
						{
							gTriggerCounter = clipTimePtr[i].trigger;
							val1 = clipTimePtr[i].time;
//							vPtr = (unsigned long *)&aa[0];
//							vPtr = (unsigned long *)&aa[4];
//							*(unsigned long *)vPtr = sdram[(gFirstFreeRam / 4) + (j * 3) + 1];
//							vPtr = (unsigned long *)&aa[8];
//							*(unsigned long *)vPtr = sdram[(gFirstFreeRam / 4) + (j * 3) + 2];

//							vPtr = (char *)&gTriggerCounter;
//							for (i = 0; i < 4; i++) *(char *)vPtr++ = aa[i];

//							vPtr = (char *)&testCnt;
//							for (i = 4; i < 12; i++) *(char *)vPtr++ = aa[i];

							Printf( "%3d   trig: %3ld; time: %ld\n", i, gTriggerCounter, val1 );
						}
*/
						break;

					case SETTRIGPOL_CMD:
//						ptr = strtok( NULL, " \t\r\n" );
//						error = SetTriggerPolarity( ptr );
						break;

					case CLIP_CMD:
						if (gMFlag)
						{
							error = CALCULATION_RUNNING;
							break;
						}
						if (gClipCounter == 0UL)
						{
							Prints( " no clipping occurred\n" );
							break;
						}
						ptr = strtok( NULL, " \t\r\n" );
						if (ptr == NULL)
						{
							val1 = 1;
							val2 = gClipCounter;
						}
						else
						{
							val1 = strtol( ptr, NULL, 10 );
							ptr = strtok( NULL, " \t\r\n" );
							if (ptr == NULL)
							{
								val2 = val1;
							}
							else
							{
								val2 = strtol( ptr, NULL, 10 );
							}
						}

						if (val1 < 1) val1 = 1;
						if (val2 > gClipCounter) val2 = gClipCounter;
						Printf( " %ld clippings occurred\n", gClipCounter );
						for (i = val1; i <= val2; i++)
						{
//							Printf( " %3d   trig: %3ld; time: %ld\n", i, clipTimePtr[i-1].trigger, clipTimePtr[i-1].time );
							MakeTimeString( string, clipTimePtr[i-1].time );
							Printf( " %d; trigger no.: %d; time: %s\n", i, clipTimePtr[i-1].trigger, string );
						}
						break;

					case GET_ERROR_CMD:
						break;

					case GMAX_CMD:
						ptr = strtok( NULL, " \t\r\n" );
						if (ptr == NULL)
						{
							Dbl2Str( string, gSettings.gMax );
							dValue = 1.0 / gSettings.gMax;
							Dbl2Str( string1, dValue );
							Printf( " Gmax for alpha synapse: %sS (%sOhm)\n", string, string1 );
							break;
						}
						if (gMFlag)
						{
							error = CALCULATION_RUNNING;
							break;
						}
						dValue = strtod( ptr, &restPtr );
						error = Str2Dbl( restPtr, &dValue );
						if (error)
						{
							error = ERR_RANGE;
							break;
						}
						gSettings.gMax = dValue;

						Dbl2Str( string, gSettings.gMax );
						dValue = 1.0 / gSettings.gMax;
						Dbl2Str( string1, dValue );
						Printf( " Gmax set to %sS (%sOhm)\n", string, string1 );
						gMltFlag = 1;
						error = SaveSettings();
						break;

					case TPEAK_CMD:
						ptr = strtok( NULL, " \t\r\n" );
						if (ptr == NULL)
						{
							Dbl2Str( string, gSettings.tPeak );
							Printf( " t peak for alpha synapse: %ss\n", string );
							break;
						}
						if (gMFlag)
						{
							error = CALCULATION_RUNNING;
							break;
						}
						dValue = strtod( ptr, &restPtr );
						error = Str2Dbl( restPtr, &dValue );
						if (error)
						{
							error = ERR_RANGE;
							break;
						}
						gSettings.tPeak = dValue;

						Dbl2Str( string, gSettings.tPeak );
						Printf( " t peak set to %ss\n", string );
						gMltFlag = 1;
						error = SaveSettings();
						break;

					case EREV_CMD:
						ptr = strtok( NULL, " \t\r\n" );
						if (ptr == NULL)
						{
							Dbl2Str( string, gSettings.eRev );
							Printf( " E reversal: %sV\n", string );
							break;
						}
						if (gMFlag)
						{
							error = CALCULATION_RUNNING;
							break;
						}
						dValue = strtod( ptr, &restPtr );
						error = Str2Dbl( restPtr, &dValue );
						if (error)
						{
							error = ERR_RANGE;
							break;
						}
						gSettings.eRev = dValue;

						Dbl2Str( string, gSettings.eRev );
						Printf( " E reversal set to %sV\n", string );
						gMltFlag = 1;
						error = SaveSettings();
						break;

					case GPEAK_CMD:
						ptr = strtok( NULL, " \t\r\n" );
						if (ptr == NULL)
						{
							Dbl2Str( string, gSettings.gPeak );
							dValue = 1.0 / gSettings.gPeak;
							Dbl2Str( string1, dValue );
							Printf( " G const for double exponential synapse: %sS (%sOhm)\n", string, string1 );
							break;
						}
						if (gMFlag)
						{
							error = CALCULATION_RUNNING;
							break;
						}
						dValue = strtod( ptr, &restPtr );
						error = Str2Dbl( restPtr, &dValue );
						if (error)
						{
							error = ERR_RANGE;
							break;
						}
						gSettings.gPeak = dValue;

						Dbl2Str( string, gSettings.gPeak );
						dValue = 1.0 / gSettings.gPeak;
						Dbl2Str( string1, dValue );
						Printf( " G const set to %sS (%sOhm)\n", string, string1 );
						gMltFlag = 1;
						error = SaveSettings();
						break;

					case TAUR_CMD:
						ptr = strtok( NULL, " \t\r\n" );
						if (ptr == NULL)
						{
							Dbl2Str( string, gSettings.tauR );
							Printf( " tau rising for double exponential synapse: %ss\n", string );
							break;
						}
						if (gMFlag)
						{
							error = CALCULATION_RUNNING;
							break;
						}
						dValue = strtod( ptr, &restPtr );
						error = Str2Dbl( restPtr, &dValue );
						if (error)
						{
							error = ERR_RANGE;
							break;
						}
						gSettings.tauR = dValue;

						Dbl2Str( string, gSettings.tauR );
						Printf( " tau rising set to %ss\n", string );
						gMltFlag = 1;
						error = SaveSettings();
						break;

					case TAUD_CMD:
						ptr = strtok( NULL, " \t\r\n" );
						if (ptr == NULL)
						{
							Dbl2Str( string, gSettings.tauD );
							Printf( " tau decay for double exponential synapse: %ss\n", string );
							break;
						}
						if (gMFlag)
						{
							error = CALCULATION_RUNNING;
							break;
						}
						dValue = strtod( ptr, &restPtr );
						error = Str2Dbl( restPtr, &dValue );
						if (error)
						{
							error = ERR_RANGE;
							break;
						}
						gSettings.tauD = dValue;

						Dbl2Str( string, gSettings.tauD );
						Printf( " tau decay set to %ss\n", string );
						gMltFlag = 1;
						error = SaveSettings();
						break;

					case NSITE_CMD:
						ptr = strtok( NULL, " \t\r\n" );
						if (ptr == NULL)
						{
							Printf( " number of release sites: %ld\n", gSettings.numRS );
							break;
						}
						if (gMFlag)
						{
							error = CALCULATION_RUNNING;
							break;
						}
						gSettings.numRS = strtol( ptr, NULL, 10 );

						Printf( " number of release sites set to %ld\n", gSettings.numRS );
						gMltFlag = 1;
						error = SaveSettings();
						break;

					case PROB_CMD:
						ptr = strtok( NULL, " \t\r\n" );
						if (ptr == NULL)
						{
							Dbl2Str( string, gSettings.pr );
							Printf( " release probability: %s\n", string );
							break;
						}
						if (gMFlag)
						{
							error = CALCULATION_RUNNING;
							break;
						}
						dValue = strtod( ptr, &restPtr );
						error = Str2Dbl( restPtr, &dValue );
						if (error)
						{
							error = ERR_RANGE;
							break;
						}
						gSettings.pr = dValue;

						Dbl2Str( string, gSettings.pr );
						Printf( " release probability set to %s\n", string );
						gMltFlag = 1;
						error = SaveSettings();
						break;

					case SIGMA_CMD:
						ptr = strtok( NULL, " \t\r\n" );
						if (ptr == NULL)
						{
							Dbl2Str( string, gSettings.sigma );
							Printf( " mini PSR sigma: %s\n", string );
							break;
						}
						if (gMFlag)
						{
							error = CALCULATION_RUNNING;
							break;
						}
						dValue = strtod( ptr, &restPtr );
						error = Str2Dbl( restPtr, &dValue );
						if (error)
						{
							error = ERR_RANGE;
							break;
						}
						gSettings.sigma = dValue;

						Dbl2Str( string, gSettings.sigma );
						Printf( " sigma set to %s\n", string );
						gMltFlag = 1;
						error = SaveSettings();
						break;


					case TAUREC_CMD:
						ptr = strtok( NULL, " \t\r\n" );
						if (ptr == NULL)
						{
							Dbl2Str( string, gSettings.taurec );
							Printf( " recovery time constant: %ss\n", string );
							break;
						}
						if (gMFlag)
						{
							error = CALCULATION_RUNNING;
							break;
						}
						dValue = strtod( ptr, &restPtr );
						error = Str2Dbl( restPtr, &dValue );
						if (error)
						{
							error = ERR_RANGE;
							break;
						}
						gSettings.taurec = dValue;

						Dbl2Str( string, gSettings.taurec );
						Printf( " recovery time constant set to %ss\n", string );
						gMltFlag = 1;
						error = SaveSettings();
						break;

					case TAUF_CMD:
						ptr = strtok( NULL, " \t\r\n" );
						if (ptr == NULL)
						{
							Dbl2Str( string, gSettings.tauf );
							Printf( " facilitation time constant: %ss\n", string );
							break;
						}
						if (gMFlag)
						{
							error = CALCULATION_RUNNING;
							break;
						}
						dValue = strtod( ptr, &restPtr );
						error = Str2Dbl( restPtr, &dValue );
						if (error)
						{
							error = ERR_RANGE;
							break;
						}
						gSettings.tauf = dValue;

						Dbl2Str( string, gSettings.tauf );
						Printf( " facilitation time constant set to %ss\n", string );
						gMltFlag = 1;
						error = SaveSettings();
						break;

					case RANDS_CMD:
						ptr = strtok( NULL, " \t\r\n" );
//						Printf( " random number generator seeds: %d\n", gRands );
						val1 = (Get_sys_count()) & 0x7FFFFFFF;
						Printf( " random number generator seed %d\n", val1 );
//						srand( (unsigned)gRands );
						temp = ran1( &val1 );
//						temp = (double)rand() / (double)RAND_MAX;
						Printf( " first random number = %g\n", temp );
						break;

					case GUNIT_CMD:
						ptr = strtok( NULL, " \t\r\n" );
						if (ptr == NULL)
						{
							Dbl2Str( string, gSettings.gUnit );
							Printf( " PSR scaling: %s\n", string );
							break;
						}
						if (gMFlag)
						{
							error = CALCULATION_RUNNING;
							break;
						}
						dValue = strtod( ptr, &restPtr );
						error = Str2Dbl( restPtr, &dValue );
						if (error)
						{
							error = ERR_RANGE;
							break;
						}
						gSettings.gUnit = dValue;
						Dbl2Str( string, gSettings.gUnit );
						Printf( " PSR scaling set to %s\n", string );

						//MakeLookupTable();
						gMltFlag = 1;
						error = SaveSettings();
						break;

					case GSD_CMD:
						ptr = strtok( NULL, " \t\r\n" );
						if (ptr == NULL)
						{
							Dbl2Str( string, gSettings.sd );
							Printf( " sd of conductances: %sS\n", string );
							break;
						}
						if (gMFlag)
						{
							error = CALCULATION_RUNNING;
							break;
						}
						dValue = strtod( ptr, &restPtr );
						error = Str2Dbl( restPtr, &dValue );
						if (error)
						{
							error = ERR_RANGE;
							break;
						}
						gSettings.sd = dValue;
						Dbl2Str( string, gSettings.sd );
						Printf( " sd of conductances set to %sS\n", string );

						//MakeLookupTable();
						gMltFlag = 1;
						error = SaveSettings();
						break;

					case NSYN_CMD:
						ptr = strtok( NULL, " \t\r\n" );
						if (ptr == NULL)
						{
							Printf( " number of synapse: %lu\n", gSettings.nSyn );
							break;
						}
						if (gMFlag)
						{
							error = CALCULATION_RUNNING;
							break;
						}
						gSettings.nSyn = strtol( ptr, NULL, 10 );
						Printf( " number of synapse set to %lu\n", gSettings.nSyn );

//						MakeLookupTable();
						gMltFlag = 1;
						error = SaveSettings();
						break;


					case ION_CMD:
						ptr = strtok( NULL, " \t\r\n" );
						if (ptr == NULL)
						{
							Printf( " ion channel: %s;\n", ionChannels[gSettings.ionChannel] );
							break;
						}

						result = GetOptions( ptr, "h", &argc, &test );
						if (result == 0)
						{
							if (test == 1)
							{
							Prints( " the following ions are available:\n");
							for ( i = 0; ionChannels[i] != NULL; i++ )
							{
								Printf( " %s\n", ionChannels[i] );
							}
							}
							else
							{
								error = WRONG_OPTION;
							}
							break;
						}

						if (gMFlag)
						{
							error = CALCULATION_RUNNING;
							break;
						}

						ConvertToLower( ptr );
						for ( i = 0; ionChannels[i] != NULL; i++ )
						{
							if ( ( strcmp( ionChannels[i], ptr ) == 0 ) ) break;
						}

						if ( ionChannels[i] == NULL )
						{
							Prints( " wrong ion channel, please use one of the following:\n");
							for ( i = 0; ionChannels[i] != NULL; i++ )
							{
								Printf( " %s\n", ionChannels[i] );
							}
							break;
						}

						gSettings.ionChannel = i;
						Printf( " ion channel set to: %s;\n", ionChannels[gSettings.ionChannel] );

						//MakeLookupTable();
						gMltFlag = 1;
						error = SaveSettings();
						break;

					case VERSION_CMD:
						PrintIdentifier();
						break;

					case TP_CMD:
						ptr = strtok( NULL, " \t\r\n" );
						if (ptr == NULL)
						{
							Prints( " parameters for test pulse:\n" );
							Dbl2Str( string, gSettings.gTP );
							Printf( " Gtp: %sS\n", string );
							dValue = (double)gSettings.dtTP * gSettings.interval;
							Dbl2Str( string, dValue );
							Printf( " dttp: %ss\n", string );
							Dbl2Str( string, gSettings.eTP );
							Printf( " Etp: %sV\n", string );
							dValue = (double)gSettings.t0TP * gSettings.interval;
							Dbl2Str( string, dValue );
							Printf( " t0tp: %ss\n", string );
//							Printf( " \ngSettings.gTP = %g; gSettings.dtTP = %d; gSettings.eTP = %g; gSettings.t0TP = %d\n", gSettings.gTP, gSettings.dtTP, gSettings.eTP, gSettings.t0TP );
							break;
						}
						if (gMFlag)
						{
							error = CALCULATION_RUNNING;
							break;
						}

						dValue = strtod( ptr, &restPtr );
						error = Str2Dbl( restPtr, &dValue );
						if (error)
						{
							error = NO_NUMBER;
							break;
						}
						gSettings.gTP = dValue;

						ptr = strtok( NULL, " \t\r\n" );
						if (ptr == NULL)
						{
							error = LESS_ARGUMENTS;
							break;
						}
						dValue = strtod( ptr, &restPtr );
						error = Str2Dbl( restPtr, &dValue );
						if (error)
						{
							error = NO_NUMBER;
							break;
						}
						gSettings.dtTP = (unsigned long)(dValue / gSettings.interval + 0.5);

						ptr = strtok( NULL, " \t\r\n" );
						if (ptr == NULL)
						{
							error = LESS_ARGUMENTS;
							break;
						}
						dValue = strtod( ptr, &restPtr );
						error = Str2Dbl( restPtr, &dValue );
						if (error)
						{
							error = NO_NUMBER;
							break;
						}
						gSettings.eTP = dValue;

						ptr = strtok( NULL, " \t\r\n" );
						if (ptr == NULL)
						{
							error = LESS_ARGUMENTS;
							break;
						}
						dValue = strtod( ptr, &restPtr );
						error = Str2Dbl( restPtr, &dValue );
						if (error)
						{
							error = NO_NUMBER;
							break;
						}
						gSettings.t0TP = (unsigned long)(dValue / gSettings.interval + 0.5);

						Prints( " test pulse parameters set to:\n" );
						Dbl2Str( string, gSettings.gTP );
						Printf( " Gtp: %sS\n", string );
						dValue = (double)gSettings.dtTP * gSettings.interval;
						Dbl2Str( string, dValue );
						Printf( " dttp: %ss\n", string );
						Dbl2Str( string, gSettings.eTP );
						Printf( " Etp: %sV\n", string );
						dValue = (double)gSettings.t0TP * gSettings.interval;
						Dbl2Str( string, dValue );
						Printf( " t0tp: %ss\n", string );
//						Printf( " \ngSettings.gTP = %g; gSettings.dtTP = %d; gSettings.eTP = %g; gSettings.t0TP = %d\n", gSettings.gTP, gSettings.dtTP, gSettings.eTP, gSettings.t0TP );

						if (gSettings.ionChannel == kTp)
						{
							gNumValues = gSettings.t0TP + gSettings.dtTP;
							gFirstValue = gSettings.t0TP;
							Printf( " gFirstValue = %d gNumValues = %d\n", gFirstValue, gNumValues );
						}
						error = SaveSettings();
						break;

					case GTP_CMD:
						ptr = strtok( NULL, " \t\r\n" );
						if (ptr == NULL)
						{
							Dbl2Str( string, gSettings.gTP );
							Printf( " Gtp: %sS\n", string );
							break;
						}
						if (gMFlag)
						{
							error = CALCULATION_RUNNING;
							break;
						}

						dValue = strtod( ptr, &restPtr );
						error = Str2Dbl( restPtr, &dValue );
						if (error)
						{
							error = NO_NUMBER;
							break;
						}
						gSettings.gTP = dValue;
						Dbl2Str( string, gSettings.gTP );
						Printf( " Gtp set to: %sS\n", string );
						error = SaveSettings();
						break;

					case DTTP_CMD:
						ptr = strtok( NULL, " \t\r\n" );
						if (ptr == NULL)
						{
							dValue = (double)gSettings.dtTP * gSettings.interval;
							Dbl2Str( string, dValue );
							Printf( " dttp: %ss\n", string );
							break;
						}
						if (gMFlag)
						{
							error = CALCULATION_RUNNING;
							break;
						}

						dValue = strtod( ptr, &restPtr );
						error = Str2Dbl( restPtr, &dValue );
						if (error)
						{
							error = NO_NUMBER;
							break;
						}

						gSettings.dtTP = (unsigned long)(dValue / gSettings.interval + 0.5);
						dValue = (double)gSettings.dtTP * gSettings.interval;
						Dbl2Str( string, dValue );
						Printf( " dttp set to: %ss\n", string );

						if (gSettings.ionChannel == kTp)
						{
							gNumValues = gSettings.t0TP + gSettings.dtTP;
							gFirstValue = gSettings.t0TP;
							Printf( " gFirstValue = %d gNumValues = %d\n", gFirstValue, gNumValues );
						}
						error = SaveSettings();
						break;

					case ETP_CMD:
						ptr = strtok( NULL, " \t\r\n" );
						if (ptr == NULL)
						{
							Dbl2Str( string, gSettings.eTP );
							Printf( " Etp: %sV\n", string );
							break;
						}
						if (gMFlag)
						{
							error = CALCULATION_RUNNING;
							break;
						}

						dValue = strtod( ptr, &restPtr );
						error = Str2Dbl( restPtr, &dValue );
						if (error)
						{
							error = NO_NUMBER;
							break;
						}

						gSettings.eTP = dValue;
						Dbl2Str( string, gSettings.eTP );
						Printf( " Etp set to: %sV\n", string );
						error = SaveSettings();
						break;

					case T0TP_CMD:
						ptr = strtok( NULL, " \t\r\n" );
						if (ptr == NULL)
						{
							dValue = (double)gSettings.t0TP * gSettings.interval;
							Dbl2Str( string, dValue );
							Printf( " t0tp: %ss\n", string );
							break;
						}
						if (gMFlag)
						{
							error = CALCULATION_RUNNING;
							break;
						}

						dValue = strtod( ptr, &restPtr );
						error = Str2Dbl( restPtr, &dValue );
						if (error)
						{
							error = NO_NUMBER;
							break;
						}

						gSettings.t0TP = (unsigned long)(dValue / gSettings.interval + 0.5);
						dValue = (double)gSettings.t0TP * gSettings.interval;
						Dbl2Str( string, dValue );

						Printf( " t0tp set to: %ss\n", string );
						if (gSettings.ionChannel == kTp)
						{
							gNumValues = gSettings.t0TP + gSettings.dtTP;
							gFirstValue = gSettings.t0TP;
							Printf( " gFirstValue = %d gNumValues = %d\n", gFirstValue, gNumValues );
						}
						error = SaveSettings();
						break;

					case GABA_ALPHA_CMD:
						ptr = strtok( NULL, " \t\r\n" );
						if (ptr == NULL)
						{
							Prints( " parameters for gaba alpha:\n" );
							Dbl2Str( string, gSettings.tPeak );
							Printf( " tpeak: %ss\n", string );
							Dbl2Str( string, gSettings.gMax );
							Printf( " gmax: %sS\n", string );
							Dbl2Str( string, gSettings.eRev );
							Printf( " erev: %sV\n", string );
							break;
						}

						if (gMFlag)
						{
							error = CALCULATION_RUNNING;
							break;
						}

						dValue = strtod( ptr, &restPtr );
						error = Str2Dbl( restPtr, &dValue );
						if (error)
						{
							error = NO_NUMBER;
							break;
						}
						gSettings.tPeak = dValue;

						ptr = strtok( NULL, " \t\r\n" );
						if (ptr == NULL)
						{
							error = LESS_ARGUMENTS;
							break;
						}
						dValue = strtod( ptr, &restPtr );
						error = Str2Dbl( restPtr, &dValue );
						if (error)
						{
							error = NO_NUMBER;
							break;
						}
						gSettings.gMax = dValue;

						ptr = strtok( NULL, " \t\r\n" );
						if (ptr == NULL)
						{
							error = LESS_ARGUMENTS;
							break;
						}
						dValue = strtod( ptr, &restPtr );
						error = Str2Dbl( restPtr, &dValue );
						if (error)
						{
							error = NO_NUMBER;
							break;
						}
						gSettings.eRev = dValue;

						gSettings.ionChannel = kGaba_alpha;

						Prints( " gaba alpha parameters set to:\n" );
						Dbl2Str( string, gSettings.tPeak );
						Printf( " tpeak: %ss\n", string );
						Dbl2Str( string, gSettings.gMax );
						Printf( " Gmax: %sS\n", string );
						Dbl2Str( string, gSettings.eRev );
						Printf( " erev: %sV\n", string );

						//MakeLookupTable();
						gMltFlag = 1;
						error = SaveSettings();
						break;

					case GABA_2EXP_CMD:
						ptr = strtok( NULL, " \t\r\n" );
						if (ptr == NULL)
						{
							Prints( " parameters for gaba alpha:\n" );
							Dbl2Str( string, gSettings.gPeak );
							Printf( " Gpeak: %sS\n", string );
							Dbl2Str( string, gSettings.tauR );
							Printf( " taur: %ss\n", string );
							Dbl2Str( string, gSettings.tauD );
							Printf( " taud: %ss\n", string );
							Dbl2Str( string, gSettings.eRev );
							Printf( " erev: %sV\n", string );
							break;
						}
						if (gMFlag)
						{
							error = CALCULATION_RUNNING;
							break;
						}

						dValue = strtod( ptr, &restPtr );
						error = Str2Dbl( restPtr, &dValue );
						if (error)
						{
							error = NO_NUMBER;
							break;
						}
						gSettings.gPeak = dValue;

						ptr = strtok( NULL, " \t\r\n" );
						if (ptr == NULL)
						{
							error = LESS_ARGUMENTS;
							break;
						}
						dValue = strtod( ptr, &restPtr );
						error = Str2Dbl( restPtr, &dValue );
						if (error)
						{
							error = NO_NUMBER;
							break;
						}
						gSettings.tauR = dValue;

						ptr = strtok( NULL, " \t\r\n" );
						if (ptr == NULL)
						{
							error = LESS_ARGUMENTS;
							break;
						}
						dValue = strtod( ptr, &restPtr );
						error = Str2Dbl( restPtr, &dValue );
						if (error)
						{
							error = NO_NUMBER;
							break;
						}
						gSettings.tauD = dValue;

						ptr = strtok( NULL, " \t\r\n" );
						if (ptr == NULL)
						{
							error = LESS_ARGUMENTS;
							break;
						}
						dValue = strtod( ptr, &restPtr );
						error = Str2Dbl( restPtr, &dValue );
						if (error)
						{
							error = NO_NUMBER;
							break;
						}
						gSettings.eRev = dValue;

						gSettings.ionChannel = kGaba_2exp;

						Prints( " gaba 2exp parameters set to:\n" );
						Dbl2Str( string, gSettings.gPeak );
						Printf( " Gpeak: %sS\n", string );
						Dbl2Str( string, gSettings.tauR );
						Printf( " tau rising: %ss\n", string );
						Dbl2Str( string, gSettings.tauD );
						Printf( " tau decay: %ss\n", string );
						Dbl2Str( string, gSettings.eRev );
						Printf( " erev: %sV\n", string );

						//MakeLookupTable();
						gMltFlag = 1;
						error = SaveSettings();
						break;



					case DYNAMIC_CMD:
						ptr = strtok( NULL, " \t\r\n" );
						if (ptr == NULL)
						{
							Prints( " parameters for dynamic alpha:\n" );
							Dbl2Str( string, gSettings.tPeak );
							Printf( " tpeak: %ss\n", string );
							Printf( " nsite: %ld\n", gSettings.numRS );
//							Dbl2Str( string, gSettings.pr );
//							Printf( " prob: %s\n", string );
							Printf( " prob: %g\n", gSettings.pr );
//							Dbl2Str( string, gSettings.sigma );
//							Printf( " sigma: %s\n", string );
							Printf( " sigma: %g\n", gSettings.sigma );
							Dbl2Str( string, gSettings.taurec);
							Printf( " taurec: %ss\n", string );
							Dbl2Str( string, gSettings.tauf);
							Printf( " tau_f: %ss\n", string );
							Dbl2Str( string, gSettings.eRev );
							Printf( " erev: %sV\n", string );
							Dbl2Str( string, gSettings.gUnit );
							Printf( " gunit: %s\n", string );
							break;
						}

						if (gMFlag)
						{
							error = CALCULATION_RUNNING;
							break;
						}

						dValue = strtod( ptr, &restPtr );
						error = Str2Dbl( restPtr, &dValue );
						if (error)
						{
							error = NO_NUMBER;
							break;
						}
						gSettings.tPeak = dValue;

						ptr = strtok( NULL, " \t\r\n" );
						if (ptr == NULL)
						{
							error = LESS_ARGUMENTS;
							break;
						}
						gSettings.numRS = strtol( ptr, NULL, 10 );

						ptr = strtok( NULL, " \t\r\n" );
						if (ptr == NULL)
						{
						error = LESS_ARGUMENTS;
							break;
						}
						dValue = strtod( ptr, &restPtr );
						error = Str2Dbl( restPtr, &dValue );
						if (error)
						{
							error = NO_NUMBER;
							break;
						}
						gSettings.pr = dValue;

						ptr = strtok( NULL, " \t\r\n" );
						if (ptr == NULL)
						{
							error = LESS_ARGUMENTS;
							break;
						}
						dValue = strtod( ptr, &restPtr );
						error = Str2Dbl( restPtr, &dValue );
						if (error)
						{
							error = NO_NUMBER;
							break;
						}
						gSettings.sigma = dValue;

						ptr = strtok( NULL, " \t\r\n" );
						if (ptr == NULL)
						{
							error = LESS_ARGUMENTS;
							break;
						}
						dValue = strtod( ptr, &restPtr );
						error = Str2Dbl( restPtr, &dValue );
						if (error)
						{
							error = NO_NUMBER;
							break;
						}
						gSettings.taurec = dValue;

						ptr = strtok( NULL, " \t\r\n" );
						if (ptr == NULL)
						{
							error = LESS_ARGUMENTS;
							break;
						}
						dValue = strtod( ptr, &restPtr );
						error = Str2Dbl( restPtr, &dValue );
						if (error)
						{
							error = NO_NUMBER;
							break;
						}
						gSettings.tauf = dValue;


						ptr = strtok( NULL, " \t\r\n" );
						if (ptr == NULL)
						{
							error = LESS_ARGUMENTS;
							break;
						}
						dValue = strtod( ptr, &restPtr );
						error = Str2Dbl( restPtr, &dValue );
						if (error)
						{
							error = NO_NUMBER;
							break;
						}
						gSettings.eRev = dValue;

						ptr = strtok( NULL, " \t\r\n" );
						if (ptr == NULL)
						{
							error = LESS_ARGUMENTS;
							break;
						}
						dValue = strtod( ptr, &restPtr );
						error = Str2Dbl( restPtr, &dValue );
						if (error)
						{
							error = NO_NUMBER;
							break;
						}
						gSettings.gUnit = dValue;
						gSettings.ionChannel = kDynamic;

						Prints( " dynamic alpha parameters set to:\n" );
						Dbl2Str( string, gSettings.tPeak );
						Printf( " tpeak: %ss\n", string );
						Printf( " N: %ld\n", gSettings.numRS );
//						Dbl2Str( string, gSettings.pr );
//						Printf( " prob: %s\n", string );
						Printf( " prob: %g\n", gSettings.pr );
//						Dbl2Str( string, gSettings.sigma );
//						Printf( " sigma: %s\n", string );
						Printf( " sigma: %g\n", gSettings.sigma );
						Dbl2Str( string, gSettings.taurec );
						Printf( " tau_rec: %ss\n", string );
						Dbl2Str( string, gSettings.tauf );
						Printf( " tau_f: %ss\n", string );
						Dbl2Str( string, gSettings.eRev );
						Printf( " erev: %sV\n", string );
						Dbl2Str( string, gSettings.gUnit );
						Printf( " g_unit: %s\n", string );

						gMltFlag = 1;
						error = SaveSettings();
						break;

					case AMPA_CMD:
						ptr = strtok( NULL, " \t\r\n" );
						if (ptr == NULL)
						{
							Prints( " parameters for ampa synapse:\n" );
							Dbl2Str( string, gSettings.gMax );
							Printf( " gmax: %sS\n", string );
							Dbl2Str( string, gSettings.sd );
							Printf( " sd: %sS\n", string );
							Dbl2Str( string, gSettings.tauR );
							Printf( " taur: %ss\n", string );
							Dbl2Str( string, gSettings.tauD );
							Printf( " taud: %ss\n", string );
							Dbl2Str( string, gSettings.eRev );
							Printf( " erev: %sV\n", string );
							Printf( " nsyn: %lu\n", gSettings.nSyn );
							break;
						}
						if (gMFlag)
						{
							error = CALCULATION_RUNNING;
							break;
						}

						dValue = strtod( ptr, &restPtr );
						error = Str2Dbl( restPtr, &dValue );
						if (error)
						{
							error = NO_NUMBER;
							break;
						}
						gSettings.gMax = dValue;

						ptr = strtok( NULL, " \t\r\n" );
						if (ptr == NULL)
						{
							error = LESS_ARGUMENTS;
							break;
						}
						dValue = strtod( ptr, &restPtr );
						error = Str2Dbl( restPtr, &dValue );
						if (error)
						{
							error = NO_NUMBER;
							break;
						}
						gSettings.sd = dValue;

						ptr = strtok( NULL, " \t\r\n" );
						if (ptr == NULL)
						{
							error = LESS_ARGUMENTS;
							break;
						}
						dValue = strtod( ptr, &restPtr );
						error = Str2Dbl( restPtr, &dValue );
						if (error)
						{
							error = NO_NUMBER;
							break;
						}
						gSettings.tauR = dValue;

						ptr = strtok( NULL, " \t\r\n" );
						if (ptr == NULL)
						{
							error = LESS_ARGUMENTS;
							break;
						}
						dValue = strtod( ptr, &restPtr );
						error = Str2Dbl( restPtr, &dValue );
						if (error)
						{
							error = NO_NUMBER;
							break;
						}
						gSettings.tauD = dValue;

						ptr = strtok( NULL, " \t\r\n" );
						if (ptr == NULL)
						{
							error = LESS_ARGUMENTS;
							break;
						}
						dValue = strtod( ptr, &restPtr );
						error = Str2Dbl( restPtr, &dValue );
						if (error)
						{
							error = NO_NUMBER;
							break;
						}
						gSettings.eRev = dValue;

						ptr = strtok( NULL, " \t\r\n" );
						if (ptr == NULL)
						{
							error = LESS_ARGUMENTS;
							break;
						}
						gSettings.nSyn = strtol( ptr, NULL, 10 );

						gSettings.ionChannel = kAmpa;

						Prints( " ampa 2exp parameters set to:\n" );
						Dbl2Str( string, gSettings.gMax );
						Printf( " gmax: %sS\n", string );
						Dbl2Str( string, gSettings.sd);
						Printf( " sd: %sS\n", string );
						Dbl2Str( string, gSettings.tauR );
						Printf( " tau rising: %ss\n", string );
						Dbl2Str( string, gSettings.tauD );
						Printf( " tau decay: %ss\n", string );
						Dbl2Str( string, gSettings.eRev );
						Printf( " erev: %sV\n", string );
						Printf( " nsyn: %lu\n", gSettings.nSyn );

						gMltFlag = 1;
						error = SaveSettings();
						break;




					case INIT_CMD:
						InitSettings( 1 );
						error = InitTimeValues();
						gMltFlag = 1;
						if (error)
						{
							PrintError( "ERROR while init time values", error );
							break;
						}

						Prints( " settings set to default values\n" );
						break;


/*********************************************************************************************************/
						// this is a "mount" command
						case MOUNT_CMD:
							ptr = strtok( NULL, " \t\r\n" );
							if (gMFlag)
							{
								error = CALCULATION_RUNNING;
								break;
							}
							if (ptr == NULL)
							{
								// Display error message.
								PrintError( "", ERR_DRIVE );
								break;
							}
							ConvertToLower( ptr );

							// Get drive number
							k = ptr[0] - 'a';
							if ((error = Mount( k )) == 0)
							{
								Printf( "drive %c: mounted\n");
							}
							break;

						// this is a "fat" information command
						case FAT_CMD:
							// Regarding the partition type :
							switch (nav_partition_type())
							{
							case FS_TYPE_FAT_12:
								// Display FAT12.
								part_type = " Drive uses FAT12\n";
								break;
							case FS_TYPE_FAT_16:
								// Display FAT16.
								part_type = " Drive uses FAT16\n";
								break;
							case FS_TYPE_FAT_32:
								// Display FAT32.
								part_type = " Drive uses FAT32\n";
								break;
							default:
								// Display error message.
								part_type = " Drive uses an unknown partition type\n";
								break;
							}
							Prints( part_type );
							break;

						// this is a "ls" command
						case LS_CMD:
							ptr = strtok( NULL, " \t\r\n" );
							// Check if params are correct or mount needed.
							if (nav_drive_get() >= nav_drive_nb() || first_ls)
							{
								first_ls = FALSE;
								// Reset navigators .
								nav_reset();
								// Use the last drive available as default.
								nav_drive_set(nav_drive_nb() - 1);
								// Mount it.
								nav_partition_mount();
							}

							// Check if the file exists
							if (ptr)
							{
								int	exists = 0;

								// reset filelist before to start the listing
								nav_filelist_reset();
								// While an item can be found
								while (nav_filelist_set(0, FS_FIND_NEXT))
								{
									// Get current item informations
									if (nav_file_name((FS_STRING)ptr, MAX_FILE_PATH_LENGTH, FS_NAME_CHECK, TRUE))
									{
										exists = 1;
										break;
									}
								}

								if (!exists)
								{
									Printf( " ls: cannot access %s: No such file or directory\n", ptr );
									break;
								}

							}

							// Get the volume name
							nav_dir_name((FS_STRING)par_str1, MAX_FILE_PATH_LENGTH);

							// Try to sort items by folders
							if (!nav_filelist_first(FS_DIR))
							{
								// Sort items by files
								nav_filelist_first(FS_FILE);
							}

							// Display general informations (drive letter and current path)
							Prints( " Volume is " );
							PutChar( 'A' + nav_drive_get() );
							Prints( ":\n Dir name is " );
							Prints( par_str1 );
							Prints( "\n" );

							// Display items informations
							Prints( "\tSize (Bytes)\tName\n");
							// reset filelist before to start the listing
							nav_filelist_reset();
							// While an item can be found
							while (nav_filelist_set(0, FS_FIND_NEXT))
							{
								// Get and display current item informations
								nav_file_name((FS_STRING)str_buff, MAX_FILE_PATH_LENGTH, FS_NAME_GET, TRUE);
								if (ptr == NULL || ((strcmp( ptr, str_buff )) == 0))
								{
									Prints( (nav_file_isdir()) ? "Dir\t" : "   \t");
									Printf( "%ld", nav_file_lgt() );
									Prints( "\t\t");
									Printf( "%s\n", str_buff );
								}
							}
							// Display the files number
							Printf( " %ld	Files\n", nav_filelist_nb(FS_FILE) );
							// Display the folders number
							Printf( " %ld	Dir\n", nav_filelist_nb(FS_DIR) );
							break;

						// this is a "cd" command
						case CD_CMD:
							ptr = strtok( NULL, " \t\r\n" );
							if (gMFlag)
							{
								error = CALCULATION_RUNNING;
								break;
							}
							if (ptr == NULL)
							{
								// Display error message.
								PrintError( "", NO_ARGUMENT );
								break;
							}

							// get arg1 length
							k = strlen(ptr);
							// Append the '/' char for the nav_setcwd to enter the chosen directory.
							if (ptr[k - 1] != '/')
							{
								ptr[k] = '/';
								ptr[k + 1] = '\0';
							}
							// Try to to set navigator on arg1 folder.
							if (!nav_setcwd((FS_STRING)ptr, TRUE, FALSE))
							{
								// Display error message.
								PrintError( "", ERR_UNKNOWN_FILE );
							}
							break;

						// this is a "cat" command
						case CAT_CMD:
							ptr = strtok( NULL, " \t\r\n" );
							if (ptr == NULL)
							{
								// Display error message.
								PrintError( "", NO_ARGUMENT );
								break;
							}

							// Try to to set navigator on arg1 file.
							if (!nav_setcwd((FS_STRING)ptr, TRUE, FALSE))
							{
								// Display error message.
								PrintError( "", ERR_UNKNOWN_FILE );
							}
							else
							{
								// Open the file.
								file_open(FOPEN_MODE_R);
								// While the end isn't reached
								while (!file_eof())
								{
									// Display next char from file.
									PutChar( file_getc() );
								}
								// Put the "end of transmission" character
								PutChar( EOT );
								// Close the file.
								file_close();
								PutChar( '\n' );
							}
							break;

						// this is a "mkdir" command
						case MKDIR_CMD:
							ptr = strtok( NULL, " \t\r\n" );
							if (gMFlag)
							{
								error = CALCULATION_RUNNING;
								break;
							}
							if (ptr == NULL)
							{
								// Display error message.
								PrintError( "", NO_ARGUMENT );
								break;
							}

							// Create the folder;
							nav_dir_make((FS_STRING)ptr);
							break;

						// this is a "touch" command
						case TOUCH_CMD:
							ptr = strtok( NULL, " \t\r\n" );
							if (gMFlag)
							{
								error = CALCULATION_RUNNING;
								break;
							}
							if (ptr == NULL)
							{
								// Display error message.
								PrintError( "", NO_ARGUMENT );
								break;
							}

							// Create the file.
							if (!nav_file_create((FS_STRING)ptr))
							{
								PrintError( "", ERR_CREATE );
							}
							break;

						// this is a "rm" command
						case RM_CMD:
							ptr = strtok( NULL, " \t\r\n" );
							if (gMFlag)
							{
								error = CALCULATION_RUNNING;
								break;
							}
							if (ptr == NULL)
							{
								// Display error message.
								PrintError( "", NO_ARGUMENT );
								break;
							}

							error = RemoveFile( ptr );
							break;

						// this is a "append"	command: Append a char to selected file.
						case APPEND_CMD:
							ptr = strtok( NULL, " \t\r\n" );
							if (gMFlag)
							{
								error = CALCULATION_RUNNING;
								break;
							}
							if (ptr == NULL)
							{
								// Display error message.
								PrintError( "", NO_ARGUMENT );
								break;
							}

							// Try to to set navigator on arg1 file.
							if (!nav_setcwd((FS_STRING)ptr, TRUE, TRUE))
							{
								// Display error message.
								PrintError( "", ERR_UNKNOWN_FILE );
							}
							else
							{
								// File exists, open it in append mode
								file_open(FOPEN_MODE_APPEND);
								// Append from USART
								AppendToFile();
								// Close the file
								file_close();
								// Display a line feed to user
								PutChar( '\n' );
							}
							break;

						// this is a "disk" command
						case NB_DRIVE_CMD:
							// Display number of drives.
							Prints( " Nb Drive(s): ");
							PutChar( '0' + nav_drive_nb() );
							PutChar( '\n' );
							break;

						// this is a "mark" command
						case SET_ID_CMD:
							// get marked index from current navigator location
							mark_index = nav_getindex();
							break;

						// this is a "goto" command
						case GOTO_ID_CMD:
							// set navigator to the marked index
							nav_gotoindex(&mark_index);
							break;

						// this is a "cp" command: Copy file to other location.
						case CP_CMD:
							if (gMFlag)
							{
								error = CALCULATION_RUNNING;
								break;
							}

							ptr = strtok( NULL, " \t\r\n" );
							if (ptr == NULL)
							{
								// Display error message.
								PrintError( "", NO_ARGUMENT );
								break;
							}

							// Try to set navigator on arg1 file.
							if (!nav_setcwd((FS_STRING)ptr, TRUE, FALSE))
							{
								// Display error message.
								PrintError( "", ERR_UNKNOWN_FILE );
							}
							else
							{
								// Get name of source to be used as same destination name.
								nav_file_name((FS_STRING)ptr, MAX_FILE_PATH_LENGTH, FS_NAME_GET, TRUE);
								// Get file size.
								file_size = nav_file_lgtsector();
								// Mark source.
								nav_file_copy();
								// Save current source position.
								sav_index = nav_getindex();
								// Goto destination.
								nav_gotoindex(&mark_index);
								// Free space check.
								if (nav_partition_space() > file_size)
								{
									// Paste.
									nav_file_paste_start((FS_STRING)ptr);
									// Restore previous nav position.
									nav_gotoindex(&sav_index);
									// Performs copy.
									while (nav_file_paste_state(FALSE) == COPY_BUSY);
								}
								// Restore previous nav position.
								nav_gotoindex(&sav_index);
							}
							break;

						// this is a "mv" command: Rename file.
						case MV_CMD:
							if (gMFlag)
							{
								error = CALCULATION_RUNNING;
								break;
							}

							ptr = strtok( NULL, " \t\r\n" );
							if (ptr == NULL)
							{
								// Display error message.
								PrintError( "", NO_ARGUMENT );
								break;
							}
							strcpy( par_str1, ptr );
							ptr = strtok( NULL, " \t\r\n" );
							if (ptr == NULL)
							{
								// Display error message.
								PrintError( "", NO_ARGUMENT );
								break;
							}

							// Save current nav position.
							sav_index = nav_getindex();
							// Try to to set navigator on arg1 folder or file.
							if (!nav_setcwd((FS_STRING)par_str1, TRUE, FALSE))
							{
								// Display error message.
								PrintError( "", ERR_UNKNOWN_FILE );
							}
							// Try to rename the file
							else if (!nav_file_rename((FS_STRING)ptr))
							{
								// Display error message.
								PrintError( "", ERR_MV );
							}
							// Restore nav position.
							nav_gotoindex(&sav_index);
							break;

						// this is a "df" command: Display free space information for all connected drives.
						case DF_CMD:
							// Save current nav position.
							sav_index = nav_getindex();
							// For all available drives :
							for (k = 0; k < nav_drive_nb(); k++)
							{
								// Select drive.
								nav_drive_set(k);
								// Try to mount.
								if (nav_partition_mount())
								{
									Prints( " " );
									// Display memory name.
									Prints( mem_name(k) );
									// Display drive letter name.
									Prints( " (");
									PutChar( 'A' + k);
									Prints( ":) Free Space: ");
									// Display free space.
									Printf( "%ld Bytes / ", nav_partition_freespace() << FS_SHIFT_B_TO_SECTOR );
									// Display available space.
									Printf( "%ld Bytes\n", nav_partition_space() << FS_SHIFT_B_TO_SECTOR );
								}
							}
							// Restore nav position.
							nav_gotoindex(&sav_index);
							break;

						// this is a "format" command : Format disk.
						case FORMAT_CMD:
							ptr = strtok( NULL, " \t\r\n" );
							if (gMFlag)
							{
								error = CALCULATION_RUNNING;
								break;
							}
							if (ptr == NULL)
							{
								// Display error message.
								PrintError( "", NO_ARGUMENT );
								break;
							}

							// Get disk number.
							k = ptr[0] - 'a';

							// Display security message.
							Printf( " do you realy want to format drive %c? (y/n)\n", ptr[0] );
							tmpc = GetChar();
							if (tmpc == 'y')
							{
								error = Format( k, FS_FORMAT_DEFAULT );
								if (error) break;
								error = SaveSettings();
								if (error) break;
								Printf( "format of disk %c successful\n", ptr[0] );
							}
							break;

						// this is a "format32" command: Format disk as FAT 32 if possible.
						case FORMAT32_CMD:
							ptr = strtok( NULL, " \t\r\n" );
							if (gMFlag)
							{
								error = CALCULATION_RUNNING;
								break;
							}
							if (ptr == NULL)
							{
								// Display error message.
								PrintError( "", NO_ARGUMENT );
								break;
							}

							// Get disk number.
							k = ptr[0] - 'a';

							// Display security message.
							Printf( " do you realy want to format drive %c? (y/n)\n", ptr[0] );
							tmpc = GetChar();
							if (tmpc == 'y')
							{
								error = Format( k, FS_FORMAT_FAT32 );
								if (error) break;
								error = SaveSettings();
								if (error) break;
								Printf( "format of disk %c successful\n", ptr[0] );
							}
							break;
						// Unknown command.

				} // switch (command)
				if (error) PrintError( "", error );
				error = 0;
			} // if (ptr != NULL)
			SetPrompt();
		} // if ((cmdBuffer = ReadLine()))

//		if (error) PrintError( "", error );


//		wdt->clr = 1;										// reset the watchdog
//		wdt_clear();

		if (gStopFlag && !gStartFlag)
		{
			gStopFlag = 0;
			gRunFlag = 0;

			gMFlag = 0;

			WriteDAC( 0, 2048 );
			WriteDAC( 1, 2048 );

			if (gClipCounter)
			{
				Printf( " %lu clippings occurred during measurement\n", gClipCounter );
//				gClip = 0;
			}
//			Prints( " all stopped\n" );
			SetPrompt();
		}

		if (gPauseFlag)
		{
			gPauseFlag = 0;
			AppendDataToFile();

			if (gDiagnosticFlag)
			{
				Printf( " triggered #%lu\n", gTriggerCounter );
			}

			if (
					gSettings.ionChannel	== kDynamic		||
					gSettings.ionChannel	== kAmpa		||
					gSettings.tMethode		== T_POISSON
				)
			{
				MakeLookupTable();
			}
/*			else
			{
				if ((gTriggerCounter % 80UL) == 0) Prints( "\n" );
				Prints( "." );
			}
*/

			SetPrompt();

			// try the interupts working
			// port PA30 and PA31 and connected
/*
			pioa->ier	=		(1 << AVR32_PIO_P31);	// enable interupt

			// change the level on the port
			if (pioa->pdsr & (1 << AVR32_PIO_P30))
			{
				pioa->codr	=	(1 << AVR32_PIO_P30);	// set output port to low
			}
			else
			{
				pioa->sodr	=	(1 << AVR32_PIO_P30);	// set output port to high
			}
*/
			error = pioa->isr;
			pioa->ier = (1 << AVR32_PIO_P21);			// enable interupt
			if (gClip)
			{
				PutChar( BEL );
				gClip = 0;
			}
		}
	} // while(1)
	return 0;
}
